C$Procedure  POLYGON  ( Check polygon-like FOV definition )

      SUBROUTINE POLYGON ( BSIGHT, VERT, N )

C$ Abstract
C
C      Checks whether parameters of polygon-like FOV are
C      presented correctly.
C
C$ Copyright
C
C     Copyright (2000), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C      TBD
C
C$ Keywords
C
C      TBD
C
C$ Declarations

      DOUBLE PRECISION      BSIGHT (3)
      DOUBLE PRECISION      VERT (3,*)
      INTEGER               N

C$ Brief_I/O
C
C      VARIABLE  I/O              DESCRIPTION
C      --------  ---  --------------------------------------------------
C      BSIGHT     I     Boresight vector of the FOV
C      VERT      I/O    Array of the FOV vertices
C      N          I     The number of the FOV vertices
C
C$ Detailed_Input
C
C      BSIGHT     Boresight vector of the FOV (not obligatory to
C                 be really unit vector)
C
C      VERT       Array of the FOV vertices, i.e. radius-vectors of
C                 the consequent FOV corners
C
C      N          The number of vertices in the FOV polygon
C
C$ Detailed_Output
C
C      VERT       Array of the FOV vertices, reordered
C                 counterclockwise if initially presented clockwise
C
C$ Parameters
C
C      None.
C
C$ Particulars
C
C     Attention! Now we begin our additional checks whether
C     POLYGON-like FOV is defined correctly.
C
C     By definition we are going to deal with convex FOVs only,
C     so the first requirement to be checked is whether the
C     given FOV is convex. We suggest the following convexity
C     check criteria:
C
C     "Direction of rotation of normals to the viewing pyramide
C     adjacent faces is to be the same for all the vertices"
C
C     Let us formulate the criteria mathematically.
C     Hereinafter we shall define "*" as a symbol of scalar
C     product whereas "x" as a symbol of cross product.
C     It is evident that normal to the viewing pyramide face
C     formed by "i" and "i+1" vertices is directed along the
C     vector
C                     _    _    _
C                     N  = V  x V    ,
C                      i    i    i+1
C             _   _
C     where   V , V    are unit vectors of the vetices.
C              i   i+1
C
C     Then the direction of face "i" rotation with respect to
C     face "i-1" (about vertex "i") is presented as follows:
C
C                            _      _     _
C                     A  = [ N    x N ] * V
C                      i      i-1    i     i
C
C     Case
C                     A  > 0
C                      i
C
C     means that the rotation is counterclockwise if viewing
C     against unit vector of "i" vertex and clockwise otherwise.
C
C     So now we can express convexity criteria as:
C     " Ai keeps the same sign for all "i" values".
C
C     For our further manipulations we prefer to deal with
C     counterclockwise presentation of consequent vertices. So
C     the routine will reorder the vertices in case of clockwise
C     definition.
C
C     The next step of the checking procedure is to verify
C     whether boresight vector is within the viewing pyramide.
C     To this end we shall construct "internal" normals to
C     the pyramide faces and check whether scalar product of
C     the boresight vector and any of the internal normals is
C     positive.
C
C$ Examples
C
C
C     1) Let our code is as follows...
C
C     CALL LATREC ( 1D0, 0D0*RPD(), 10D0*RPD(), BSIGHT)
C     NSIDES = 4
C     CALL LATREC ( 1D0, -50D0*RPD(),  10D0*RPD(), VERT(1,1) )
C     CALL LATREC ( 1D0, -50D0*RPD(), -10D0*RPD(), VERT(1,2) )
C     CALL LATREC ( 1D0,  50D0*RPD(), -10D0*RPD(), VERT(1,3) )
C     CALL LATREC ( 1D0,  50D0*RPD(),  10D0*RPD(), VERT(1,4) )
C     CALL POLYGON ( BSIGHT, VERT, NSIDES )
C
C     ... then all the checks will terminate successfully.
C
C______________________________________________________________________
C
C     2) Let our code is as follows...
C
C     CALL LATREC ( 1D0, 0D0*RPD(), 10D0*RPD(), BSIGHT)
C     NSIDES = 4
C     CALL LATREC ( 1D0, -50D0*RPD(),  10D0*RPD(), VERT(1,1) )
C     CALL LATREC ( 1D0, -50D0*RPD(), -10D0*RPD(), VERT(1,4) )
C     CALL LATREC ( 1D0,  50D0*RPD(), -10D0*RPD(), VERT(1,3) )
C     CALL LATREC ( 1D0,  50D0*RPD(),  10D0*RPD(), VERT(1,2) )
C     CALL POLYGON ( BSIGHT, VERT, NSIDES )
C
C     ... then we obtain the following results:
C
C  ................................................................
C
C  Sorry, have to reorder FOV vertices
C
C  ................................................................
C
C    ... that means that our FOV is convex and contains its
C    boresight vector but vertices are presented clockwise, so
C    the routine had to reorder the vertices to obtain counter-
C    clockwise configuration.
C
C______________________________________________________________________
C
C     3) Let our code is as follows...
C
C     CALL LATREC ( 1D0, 0D0*RPD(), 10D0*RPD(), BSIGHT)
C     NSIDES = 4
C     CALL LATREC ( 1D0, -50D0*RPD(),  10D0*RPD(), VERT(1,1) )
C     CALL LATREC ( 1D0, -50D0*RPD(), -10D0*RPD(), VERT(1,3) )
C     CALL LATREC ( 1D0,  50D0*RPD(), -10D0*RPD(), VERT(1,2) )
C     CALL LATREC ( 1D0,  50D0*RPD(),  10D0*RPD(), VERT(1,4) )
C     CALL POLYGON ( BSIGHT, VERT, NSIDES )
C
C     ... then we obtain the following results:
C
C  ................................................................
C
C  ================================================================
C
C  Toolkit version: N0049
C
C  SPICE(FOVISNOTCONVEX) --
C
C  FOV is not convex
C
C  Oh, by the way:  The SPICELIB error handling actions are
C  USER-TAILORABLE.  You can choose whether the Toolkit aborts or
C  continues when errors occur, which error messages to output,
C  and where to send the output.  Please read the ERROR "Required
C  Reading" file, or see the routines ERRACT, ERRDEV, and ERRPRT.
C
C  ================================================================
C  Return code 1
C
C______________________________________________________________________
C
C     4) Let our code is as follows...
C
C     CALL LATREC ( 1D0, 0D0*RPD(), 10D0*RPD(), BSIGHT)
C     NSIDES = 4
C     CALL LATREC ( 1D0, -150D0*RPD(),  10D0*RPD(), VERT(1,1) )
C     CALL LATREC ( 1D0, -150D0*RPD(), -10D0*RPD(), VERT(1,2) )
C     CALL LATREC ( 1D0,  150D0*RPD(), -10D0*RPD(), VERT(1,3) )
C     CALL LATREC ( 1D0,  150D0*RPD(),  10D0*RPD(), VERT(1,4) )
C     CALL POLYGON ( BSIGHT, VERT, NSIDES )
C
C     ... then we obtain the following results:
C
C  ................................................................
C
C  Sorry, have to reorder FOV vertices
C
C  ================================================================
C
C  Toolkit version: N0049
C
C  SPICE(BSIGHTOUTSIDEFOV) --
C
C  Boresight vector is outside the FOV boundaries.
C
C  Oh, by the way:  The SPICELIB error handling actions are
C  USER-TAILORABLE.  You can choose whether the Toolkit aborts or
C  continues when errors occur, which error messages to output,
C  and where to send the output.  Please read the ERROR "Required
C  Reading" file, or see the routines ERRACT, ERRDEV, and ERRPRT.C
C  ================================================================
C  Return code 1
C
C______________________________________________________________________
C
C
C$ Restrictions
C
C      None.
C
C$ Exceptions
C
C     Error free.
C
C$ Files
C
C      None.
C
C$ Literature_References
C
C      None.
C
C$ Author_and_Institution
C
C     V.N. Kheifets  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 01-DEC-1999 (VNK)
C
C-&

C$ Index_Entries
C
C     TBD
C
C-&

C
C     Local variables
C
      DOUBLE PRECISION      P (3)
      DOUBLE PRECISION      R (3)
      DOUBLE PRECISION      S (3)
      DOUBLE PRECISION      A
      INTEGER               I
      INTEGER               J
      INTEGER               K

C
C     SPICELIB functions
C
      DOUBLE PRECISION      VDOT

C
C     At first check whether input data makes sense
C
      IF ( N .LT. 3 ) THEN

         CALL SETMSG ( 'The number of vertices # is less than 3' )
         CALL ERRINT ( '#', N                                    )
         CALL SIGERR ( 'SPICE(TOOFEWVERTICES)'                   )
         RETURN

      ELSE IF ( VDOT ( BSIGHT, BSIGHT ) .LT. 1D-15 ) THEN

         CALL SETMSG ( 'The boresight vector is zero' )
         CALL SIGERR ( 'SPICE(ZEROVECTOR)'            )
         RETURN

      END IF

      DO I = 1, N

         IF ( VDOT ( VERT(1,I), VERT(1,I) ) .LT. 1D-15 ) THEN

            CALL SETMSG ( 'The vertice # vector is zero' )
            CALL ERRINT ( '#', I                         )
            CALL SIGERR ( 'SPICE(ZEROVECTOR)'            )
            RETURN

         END IF

      END DO

C
C     Step1. Check whether the FOV polygon is convex
C     --------------------------------------------------------------
C
C     Generate set of unit vectors that are normals to the
C     viewing faces
C
      DO I = 1, N

         J = I + 1

         IF ( I .EQ. N ) THEN

            J = 1

         END IF

         K = I - 1

         IF ( I .EQ. 1 ) THEN

            K = N

         END IF

         CALL VCRSS ( VERT(1,K), VERT(1,I), P )
         CALL VCRSS ( VERT(1,I), VERT(1,J), R )
         CALL VCRSS ( P, R, S )

         IF ( I .EQ. 1 ) THEN

            A = VDOT ( VERT(1,I), S )

         ELSE IF ( A * VDOT ( VERT(1,I), S ) .LT. 0 ) THEN

            CALL SETMSG ( 'FOV is not convex '   )
            CALL SIGERR ( 'SPICE(FOVISNOTCONVEX)' )
            RETURN

         END IF

      END DO

C
C     Step2. CCW checking procedure.
C     --------------------------------------------------------------
C
      IF ( A .LT. 0 ) THEN

C
C        Unfortunately the vertices are presented clockwise, so
C        we have to reorder them by ourselves...
C
         CALL TOSTDO ( 'Sorry, have to reorder FOV vertices' )
         DO I = 2, ( N - 1 ) / 2 + 1

            J = N + 2 - I
            CALL MOVED ( VERT(1,I), 3, R         )
            CALL MOVED ( VERT(1,J), 3, VERT(1,I) )
            CALL MOVED ( R,         3, VERT(1,J) )

         END DO

      END IF

C
C     Step3. Check whether the boresight vector is really within FOV
C     --------------------------------------------------------------
C
C     Generate set of unit vectors that are internal normals
C     to the viewing pyramide faces and perform required check.
C
      DO I = 1, N

         J = I + 1

         IF ( I .EQ. N ) THEN

            J = 1

         END IF

         CALL VCRSS ( VERT(1,I), VERT(1,J), R )

         IF (  VDOT ( BSIGHT, R ) .LT. 0D0 ) THEN

            CALL SETMSG ( 'Boresight vector is outside the ' //
     .                    'FOV boundaries.'                  )
            CALL SIGERR ( 'SPICE(BSIGHTOUTSIDEFOV)'          )
            RETURN

         END IF

      END DO

      END
		