C$Procedure      STAIT2 ( Add item to Type 2 Sky catalog metainfo area )

      SUBROUTINE STAIT2 ( HANDLE, NAME, NUMBER, DATA )

C$ Abstract
C
C     Add an item to a Type 2 Sky catalog meta information area.
C
C$ Copyright
C
C     Copyright (2000), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     SPICE Sky Catalog Subsystem
C
C$ Keywords
C
C     TBD
C
C$ Declarations

      IMPLICIT              NONE

      INCLUDE               'starlib.inc'      

      INTEGER               HANDLE
      CHARACTER*(*)         NAME
      INTEGER               NUMBER
      INTEGER               DATA  (*)

C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     HANDLE     I   DAS file handle 
C     NAME       I   Name of the array to be added
C     NUMBER     I   Number of elements in the array
C     DATA       I   Array data
C
C$ Detailed_Input
C
C     HANDLE         is the handle of a DAS file containing Type 2 Sky 
C                    Catalog opened for write access.
C
C     NAME           is a name of the array to be added to the meta 
C                    information area of the catalog.
C
C     NUMBER         is the number of elements in the integer
C                    array to be added to the meta information area of 
C                    the catalog. If NUMBER is 0 then no action will be 
C                    taken by this routine.
C
C     DATA           is the integer array data to be added.
C
C$ Detailed_Output
C
C     None
C
C$ Parameters
C
C     METNLN      is the maximum length of the meta information item 
C                 name.
C
C$ Exceptions
C
C     1) If number of elements in the meta information array to be 
C        added is less then zero, then the error
C        'SPICE(BADNUMBEROFELEM)' will be signalled.
C
C     2) If the length of the meta information item name is greater 
C        METLEN characters, then the error 'SPICE(NAMEISTOOLONG)'
C        will be signalled.
C
C     3) If meta information directory is full, then the error
C        'SPICE(METADIROVERFLOW)' will be signalled.
C
C     4) If last file address is not the same as the last address for 
C        the last added meta information item, then the error
C        'SPICE(IMPROPERMETAADD)' will be signalled.
C
C     5) If data cannot be written to/updated in the file, them 
C        corresponding error will be signalled by low level DAS
C        routines which this routine calls.
C
C$ Files
C
C     This routine operates on a DAS file attached to HANDLE and opened
C     for write access.
C
C     The following MUST be done BEFORE calling this routine the 
C     first time on a file:
C
C        -- the BEGMET integer numbers must be written to the file to 
C           setup complete META DIRECTORY area; (values are 
C           insignificant though zeros are desired)
C
C        -- the fist integer in the file must be updated to be 1;
C
C     The following must be done AFTER the first call to this routine
C     but BEFORE the second call to this routine on a file:
C
C        -- begin and end addresses for the data location of the meta 
C           information array added by the first call  MUST be reset to 
C           be actual addresses at which the data was written; these 
C           addresses are stored in the integers 6 and 7 in the file;
C
C     The example provided in this header illustrates required order of 
C     actions.
C
C     Each subsequent time this routine is called, it modifies the 
C     file as follows:
C
C        1) replaces the first six un-used integers in the META  
C           DIRECTORY with packed array name (four integers) and begin 
C           and end DAS addresses of the area at which the array data  
C           is stored;
C
C        2) replaces the first integer in the file with address of the
C           last used integer in the META DIRECTORY (the integer 
C           containing the last of six numbers described in 1).);
C
C        3) appends meta array data to the end of the file, i.e. after 
C           each call the file size increases by NUMBER integers.
C
C$ Particulars
C
C     This routine does two things:
C
C        -- "registers" meta information item in the meta information
C           directory;
C
C        -- adds meta information item data to the end of the meta
C           information are of the file.
C
C     At the first BEGMET-1 characters of the file are are reserved for
C     META DIRECTORY. It is intended to make it easy to augment the
C     METAINFORMATION segment with additional information during
C     modifications, to fetch the information from the segment and to
C     support fast and easy access to the guide information of the
C     catalog. We shall consider the information as a set of integer
C     arrays (scalars could be considered as arrays of 1 element). So,
C     META DIRECTORY structure is as follows:
C
C        -- N - the number of filled directory elements (one integer)
C
C        -- Meta information Item 1 (six integers)
C
C        -- Meta information Item 2 (six integers)
C
C        --   ...
C
C        -- Meta information Item N (six integers)
C
C     This diagram illustrates META DIRECTORY and METAINFORMATION 
C     block layout (each square represents one integer number; DAS 
C     address of the number is given in the top left corner of the 
C     square): 
C        
C              +-------+    N contains address of the last used
C       META   |1      |        element of the META DIRECTORY
C     DIRECTORY|   N   | <-----------------------------------------.
C              |       |                                           |
C              +-------+-------+-------+-------+-------+-------+   |
C              |2                              |6 Beg. |7 End  |   |
C              |     Name of the Array # 1     | Addr 1| Addr 1|   |
C              |                               | (BA1) | (EA1) |   |
C              +-------+-------+-------+-------+-------+-------+   |
C              |8                              |12 Beg.|13 End |   |
C              |     Name of the Array # 2     | Addr 2| Addr 2|   |
C              |                               | (BA2) | (EA2) |   |
C              +-------+-------+-------+-------+-------+-------+   |
C              |14                             |18     |19     |   |
C                 ..........................................       |
C              |                               |       |       |   |
C              +-------+-------+-------+-------+-------+-------+   |
C              |(N-1)*6+2                      |  Beg  |  End  |   |
C              |     Name of the Array # n     | Addr n| Addr n| <-'
C              |                               | (BAn) | (EAn) |   
C              +-------+-------+-------+-------+-------+-------+
C              |                               |       |       |   
C                 ..........................................   
C              |                               |       |       |   
C              +-------+-------+-------+-------+-------+-------+
C     METAINFO |BA1                     EA1    |BA2            |   
C     starts @ |      Array # 1 data           |               |   
C      BEGMET  |                               |               |   
C              +-------+-------+-------+-------+-------+-------+
C              |                                               |   
C              |                Array # 2 Data                 |   
C              |                                               |   
C              +-------+-------+-------+-------+-------+-------+
C              |         EA2   |BA3/EA3|BA4                    |   
C              |               |       |     Array # 4 Data    |   
C              |               |       |                       |   
C              +-------+-------+-------+-------+-------+-------+
C              |                                               |   
C                 ..........................................
C              |                                               |   
C              +-------+-------+-------+-------+-------+-------+
C              |        EA(n-1)|BAn             EAn    |BA(n+1)|   
C              |               |   Array # n Data      |       |   
C              |               |                       |       |   
C              +-------+-------+-------+-------+-------+-------+
C              |                                               |   
C                 ..........................................
C
C     Each of the metainformation items is presented by six integers.
C     The first four of them present name of the item in coded form,
C     the 5th and the 6th integer are the catalog file addresses 
C     corresponding to the first and to the last elements of the array.
C
C$ Examples
C
C     This example illustrates what initializations must be done prior
C     to and after the first call to STAIT2 on a new catalog file.
C
C     C
C     C     Include Sky catalog parameters.
C     C
C           INCLUDE               'starlib.inc'
C           ...
C
C     C
C     C     Open new DAS file for write access. ST2TYP is a parameter
C     C     from include file.
C     C
C           CALL DASONW ( FNAM, ST2TYP, IFNAME, NCR, HNDL )
C
C     C
C     C     Fill META DIRECTORY with zero. BEGMET is a parameter
C     C     from include file.
C     C
C           J = 0
C           DO I = 1, BEGMET - 1
C              CALL DASADI ( HANDLE, 1, J )
C           END DO
C
C     C
C     C     Update the very first integer in the file to be 1.
C     C
C           J = 1
C           CALL DASUDI ( HANDLE, 1, 1, J )
C
C     C
C     C     Add number of objects by calling STAIT2 the first time 
C     C     on this file.
C     C
C           CALL STAIT2 ( HANDLE, MDNOFO, 1, NOBJ )
C
C     C
C     C     Manually update begin and end addresses of the range 
C     C     which NOBJ was stored to what it actually is -- 
C     C     BEGMET:BEGMET.
C     C
C           CALL DASUDI ( HANDLE, 6, 6, BEGMET )
C           CALL DASUDI ( HANDLE, 7, 7, BEGMET )
C
C     C
C     C     Convert to integers and store in metainformation 
C     C     the catalog name.
C     C
C           CALL STT2I2 ( CATNAM, CATNLN, Z, J )
C           CALL STAIT2 ( HANDLE, MDCNAM, CATNLN/NCHARI, Z )
C           ...
C
C$ Restrictions
C
C     Catalog file must be opened and meta information areas and 
C     directory must be initialized by consecutive calls STWOP2 and 
C     STWCI2 entries of the STWMG2 catalog writer umbrella.
C
C     No other addition to the catalog can happen between calls 
C     to this routine.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     V.N. Kheifets   ( JPL )
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 1-DECEMBER-1999 (VNK)
C
C-&

C$ Index_Entries
C
C     write meta_information item to type_2 sky_catalog
C
C-&

C
C     SPICELIB functions
C
      INTEGER               NBLEN
      LOGICAL               RETURN

C
C     Local variables
C
      CHARACTER*(METNLN)    NAM
      INTEGER               ARRAY (6)
      INTEGER               N
      INTEGER               K
      INTEGER               LASTC
      INTEGER               LASTD
      INTEGER               LASTI

C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'STAIT2' )
      END IF

C
C     Number of elements to be added is zero ..  we don't add anything.
C
      IF ( NUMBER .EQ. 0 ) THEN
         CALL CHKOUT ( 'STAIT2' )
         RETURN
      END IF

C
C     Check whether number of elements to be added makes sense.
C
      IF ( NUMBER .LT. 0 ) THEN
         CALL SETMSG ( 'The number of elements in the meta '      //
     .                 'information array ''#'' was less than '   //
     .                 'or equal to zero (=#). This doesn''t '    //
     .                 'make sense.'                              )
         CALL ERRCH  ( '#', NAME                                  )
         CALL ERRINT ( '#', NUMBER                                )
         CALL SIGERR ( 'SPICE(BADNUMBEROFELEM)'                   )
         CALL CHKOUT ( 'STAIT2'                                   )
         RETURN
      END IF

C
C     Check whether the name of the metainformation array is not 
C     too long.
C
      IF ( NBLEN(NAME) .GT. METNLN ) THEN
         CALL SETMSG ( 'The length of the meta information item ' //
     .                 '''#'' was greater than # characters '     //
     .                 'permitted by catalog design.'             )
         CALL ERRCH  ( '#', NAME                                  )
         CALL ERRINT ( '#', METNLN                                )
         CALL SIGERR ( 'SPICE(NAMEISTOOLONG)'                     )
         CALL CHKOUT ( 'STAIT2'                                   )
         RETURN
      END IF

      NAM = NAME 

C
C     Convert the parameter name to (METNLN/NCHARI)-element integer 
C     array.
C
      CALL STT2I2 ( NAM, METNLN, ARRAY, N )

C
C     Read K - the current number of records in the catalog file
C     metainformation directory (including the record with the K 
C     value itself.)
C
      CALL DASRDI ( HANDLE, 1, 1, K )

C
C     Check whether we have any space left in the meta information
C     directory area. It's has a fixed size of BEGMET-1, and if it's
C     full we cannot add this item to it.
C
      IF ( K + 6 .GE. BEGMET ) THEN
         CALL SETMSG ( 'Item ''#'' cannot be added to the meta '  //
     .                 'information area because meta '           //
     .                 'information directory is full.'           )
         CALL ERRCH  ( '#', NAME                                  )
         CALL SIGERR ( 'SPICE(METADIROVERFLOW)'                   )
         CALL CHKOUT ( 'STAIT2'                                   )
         RETURN        
      END IF

C
C     Read N - the current number of integers in the catalog file.
C
      CALL DASRDI ( HANDLE, K, K, N )

C
C     Verify that N is actually the last integer number in the file.
C     (Note that this check accounts for the fact that initialization 
C     of the meta directory must be done by putting 1 into the first 
C     integer number of file. It's done in the STWOP2.)
C
      CALL DASLLA ( HANDLE, LASTC, LASTD, LASTI )
      IF ( K .NE. 1 .AND. LASTI .NE. N ) THEN
         CALL SETMSG ( 'The last catalog file integer address '   //
     .                 '# is not equal to the address of the '    //
     .                 'last catalog meta information area '      //
     .                 'element #. This may indicate that some '  //
     .                 'addition operation was done of the file ' //
     .                 'before building of the meta information ' //
     .                 'area was completed, or that '             //
     .                 'metainformation item addition was '       //
     .                 'was attempted after catalog data '        //
     .                 'addition has started.'                    )
         CALL ERRINT ( '#', LASTI                                 )
         CALL ERRINT ( '#', N                                     )
         CALL SIGERR ( 'SPICE(IMPROPERMETAADD)'                   )
         CALL CHKOUT ( 'STAIT2'                                   )
         RETURN        
      END IF

C
C     Complete and add a new item to the catalog file metainformation
C     directory.
C
      ARRAY(5) = N + 1
      ARRAY(6) = N + NUMBER
      CALL DASUDI ( HANDLE, K+1, K+6, ARRAY )

C
C     Update the number of records in the catalog file metainformation
C     directory.
C
      K = K + 6
      CALL DASUDI ( HANDLE, 1, 1, K )

C
C     Add array DATA containing NUMBER of integers to the catalog file.
C
      CALL DASADI ( HANDLE, NUMBER, DATA )

C
C     All done. Checking out.
C
      CALL CHKOUT( 'STAIT2' )

      RETURN

      END

