C$Procedure   STDIV2 ( Attributes of celestial sphere splitting )

      SUBROUTINE STDIV2 ( NBANDS, DELDEC, NZONES, NINBND, NSUM )

C$ Abstract
C
C     Compute the attributes of celestial sphere splitting into 
C     elementary zones for a given number of declination bands
C     using SPICE Type 2 Sky Catalog celestial sphere splitting 
C     scheme.
C
C$ Copyright
C
C     Copyright (2000), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     SPICE Sky Catalog Subsystem
C
C$ Keywords
C
C     TBD
C
C$ Declarations

      IMPLICIT              NONE

      INTEGER               NBANDS
      DOUBLE PRECISION      DELDEC
      INTEGER               NZONES
      INTEGER               NINBND (*)
      INTEGER               NSUM   (*)

C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     NBANDS     I   The number of declination bands
C     DELDEC     O   The height of each zone, in radians
C     NZONES     O   Total number of elementary zones
C     NINBND     O   Array with numbers of zones in each band
C     NSUM       O   Array with numbers of zones preceding each band
C
C$ Detailed_Input
C
C     NBANDS         is the number of declination bands on the
C                    celestial sphere.
C
C$ Detailed_Output
C
C     DELDEC         is the height -- angular extent in declination --
C                    of any individual zone in radians.
C
C     NZONES         is the total number of elementary zones on the
C                    celestial sphere.
C
C     NINBND         is an integer array of NBANDS elements
C                    where i-th element is the number of elementary
C                    zones in the i-th declination band.
C
C     NSUM           is an integer array of NBANDS elements where i-th 
C                    element is the sum of the numbers of elementary 
C                    zones in declination bands 1 through i-1.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) If the number of declination bands is less than or equal
C        to zero, then the error 'SPICE(INVALIDNUMOFBANDS)' will 
C        be signalled.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     This routine implements the scheme of sky catalog "positional"
C     organization in which the celestial sphere is divided into an
C     ordered set of "elementary zones".
C
C     The celestial sphere is subdivided to "declination bands" that
C     are bound by small circles and correspond equally spaced
C     declinations from +pi/2 to -pi/2 with constant step DELDEC. Each
C     band is subdivided into "elementary zones" by great circles of
C     right ascension. In each band this "cutting" is carried out in
C     such a way to obtain equal zones with the true angular extent in
C     width to be as close as possible to the angular in height.
C
C     The band indexing implemented by this routine is as follows: the
C     first north polar declination band is from pi/2-DELDEC to pi/2,
C     the second band is from pi/2-2*DELDEC to pi/2-DELDEC, and so on
C     until south polar, NBANDS-th, declination band is reached.
C
C$ Examples
C
C     In this example STDIV2 is called to compute celestial sphere 
C     splitting attributes for the case of 30-degree wide declination 
C     bands (total of 6 bands).
C
C           INTEGER               NBANDS
C           PARAMETER           ( NBANDS = 6 )
C     
C           DOUBLE PRECISION      DELDEC
C           INTEGER               NZONES
C           INTEGER               NINBND ( NBANDS )
C           INTEGER               NSUM   ( NBANDS )
C           INTEGER               I
C     
C           DOUBLE PRECISION      DPR
C     
C           CALL STDIV2( NBANDS,  DELDEC, NZONES, NINBND, NSUM )
C     
C           WRITE (*,*) ' '
C           WRITE (*,*) '   Number of DEC Bands = ', NBANDS
C           WRITE (*,*) '       DEC Band Height = ', DELDEC*DPR()
C           WRITE (*,*) ' Total Number of Zones = ', NZONES
C           WRITE (*,*) ' '
C           WRITE (*,*) '  Band    Number of Zones   Number of Zones '
C           WRITE (*,*) ' Index       In Band        Preceding Band '
C           WRITE (*,*) '-------   ---------------   --------------- '
C           DO I = 1, NBANDS
C              WRITE (*,'(I4,2I16)') I, NINBND(I), NSUM(I)
C           END DO
C           WRITE (*,*) ' '
C     
C           END
C
C     The example produces the following output:
C
C         Number of DEC Bands =  6
C             DEC Band Height =   30.
C       Total Number of Zones =  44
C       
C        Band    Number of Zones   Number of Zones 
C       Index       In Band        Preceding Band 
C      -------   ---------------   --------------- 
C        1               3               0
C        2               8               3
C        3              11              11
C        4              11              22
C        5               8              33
C        6               3              41
C       
C$ Restrictions
C
C     The input number of declination bands must be greater than 
C     zero.
C
C     The output arrays must be big enough to hold the number of 
C     elements equal to the input number of declination bands. 
C
C$ Literature_References
C
C     None
C
C$ Author_and_Institution
C
C     V.N. Kheifets  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 01-DEC-1999 (VNK)
C
C-&

C$ Index_Entries
C
C     celestial sphere splitting attributes for type_2 sky_catalog
C
C-&

C
C     Local variables
C
      DOUBLE PRECISION      DEL2
      DOUBLE PRECISION      DEC1
      DOUBLE PRECISION      DEC2
      DOUBLE PRECISION      SIN1
      DOUBLE PRECISION      SIN2
      DOUBLE PRECISION      S
      INTEGER               I

C
C     SPICELIB functions
C
      DOUBLE PRECISION      PI
      LOGICAL               RETURN

C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'STDIV2' )
      END IF

C
C     "Check-in" input number of declination bands. It must be 
C     greater than zero.
C
      IF ( NBANDS .LE. 0 ) THEN
         CALL SETMSG ( 'The input number of declination bands '   //
     .                 'must be greater than zero. It was #.'     )
         CALL ERRINT ( '#', NBANDS                                )
         CALL SIGERR ( 'SPICE(INVALIDNUMOFBANDS)'                 )
         CALL CHKOUT ( 'STDIV2'                                   )
         RETURN
      END IF

C
C     Determine the height of each declination band.
C
      DELDEC = PI() / NBANDS

C
C     Initialize variables holding geometry -- compute target average 
C     elementary zone area, set upper angular extent of the first
C     zone to be PI/2 (north pole) and take its SIN.
C
      DEL2    = DELDEC**2
      DEC1    = PI() / 2.0D0
      SIN1    = 1.0D0

C
C     Initialize counters.
C
      NZONES  = 0
      NSUM(1) = 0

C
C     Iterate to compute zone numbers for each band.
C
      DO I = 1, NBANDS

C
C        Compute lower angular boundary of the current declination 
C        band -- we go from the north pole to the south pole.  Also, 
C        take lower extents' SIN.
C
         DEC2 = DEC1 - DELDEC
         SIN2 = DSIN(DEC2)

C
C        Compute S -- the surface area of current declination band
C        bound by upper and lower extents.
C
         S = 2D0 * PI() * ( SIN1 - SIN2 )

C
C        Compute how many zones of the target size fit into the
C        current band. Increase total zone count by this number.
C
         NINBND(I) = S / DEL2 + .5D0
         NZONES = NZONES + NINBND(I)

C
C        If it not the first zone, compute cumulative count of zones
C        in all band preceding the current band.
C
         IF ( I .GT. 1 ) THEN 
            NSUM(I) = NSUM(I-1) + NINBND(I-1)
         END IF

C
C        Assign the lower extent to be the upper extent of the next 
C        zone.
C
         DEC1 = DEC2
         SIN1 = SIN2

      END DO

      CALL CHKOUT ( 'STDIV2' )

      RETURN

      END

