C$Procedure   STF2I2 ( Packing few integer numbers into one integer )

      INTEGER FUNCTION STF2I2 ( N, A )

C$ Abstract
C
C     Pack a few integer numbers into one integer and return it.
C
C$ Copyright
C
C     Copyright (2000), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     SPICE Sky Catalog Subsystem
C
C$ Keywords
C
C     TBD
C
C$ Declarations

      IMPLICIT              NONE

      INCLUDE               'starlib.inc'

      INTEGER               N
      INTEGER               A ( * )

C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     N          I   Number of integers to be packed
C     A          I   Array of N integer numbers to be packed
C     STF2I2     O   Integer number containing packed array A
C
C$ Detailed_Input
C
C     N              is the number of integer values to be packed.
C
C     A              is an array of N integer numbers to
C                    be packed. The integer numbers must be
C                    small enough to be represented by 32/N bits.
C
C$ Detailed_Output
C
C     STF2I2         is the output integer containing all elements 
C                    of A array in packed form.
C
C$ Parameters
C
C     See include file.
C
C$ Exceptions
C     
C     1) If the number of integers to be packed is less than or 
C        equal to zero, then the error 'SPICE(TOOFEWINTEGERS)' 
C        will be signalled.
C     
C     2) If the number of integers to be packed is greater than
C        parameter NIIMAX, then the error 'SPICE(TOOMANYINTEGERS)' 
C        will be signalled.
C
C     3) If one of the input array elements is big to be represented 
C        by the number of bits dedicated to it, then the error 
C        'SPICE(VALUEISTOOLARGE)' will be signalled.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     Let we have input array A of N integer numbers to be packed
C     into one integer J (output value). Thus each element of A is
C     to be presented by M bits, where M = 32 / N. So the first
C     step is to check whether each element A(I), I=1,...,N fits
C     to the allowed zone, namely whether the following requirement
C     is met for all I from 1 through N:
C
C                    M-1
C           | A | < 2
C              i
C
C     If the requirement is violated at least for one I, the
C     computation is terminated and corresponding error message
C     is generated. Otherwise the following packing scheme is
C     carried out. 32-bit output variable J is divided into N
C     M-bit subsets without gaps in such a way that these NxM
C     bits are shifted right to the least significant bit (see
C     below):
C
C           31..............................0
C                xxx...x ... xxx...x xxx...x
C                \_____/     \_____/ \_____/
C                   M           M       M
C                  A(1)  ...  A(N-1)   A(N)
C
C     Clear variable J. For all M-bit pieces, beginning with the
C     rightest, corresponding to A(N), execute the following
C     operations:
C
C           1) B = A .
C                       i
C                                        M
C           2) If B < 0, assign B = B + 2 .
C
C                           M*(N-i)
C           3) J = J + B * 2
C
C     The operations are performed for all i but i=1. For i=1 the
C     following procedure is carried out:
C
C                                         M*(N-1)
C     If A  is not negative, J = J + A * 2       , otherwise:
C         1                           1
C
C        if remainder of 32 divided to N is equal 0, then
C
C                                 M*(N-1)
C                        J = A * 2       - J,
C                             1
C
C                                 M         M*(N-1)
C        else          J = J + ( 2 + A ) * 2
C                                     1
C
C$ Examples
C
C     Assume we have the following values of integer numbers to
C     be packed (i.e. elements of A array):
C
C             A(1)  = -3
C             A(2)  = -1
C             A(3)  =  2
C             A(4)  =  0
C             A(5)  = -5
C             A(6)  =  4
C             A(7)  = -6
C             A(8)  = -3
C
C     Then on calling
C
C             J = STF2I2 ( 8, A )
C
C     we obtain J = -1059108013 (decimal) = C0DF4B53 (hex)
C
C$ Restrictions
C
C     The number of integers to be packed must to be positive and 
C     less than 17 (or NIIMAX+1).
C
C     All elements of A array must "fit" into the allowed number of 
C     bits, namely the following requirement must be met for all i 
C     from 1 through N:
C
C                    M-1
C           | A | < 2
C              i
C
C     where 
C
C           M = 32 / N.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     V.N. Kheifets   ( JPL )
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 1-DECEMBER-1999 (VNK)
C
C-&

C$ Index_Entries
C
C     pack few integer numbers into one for type_2 sky_catalog
C
C-&

C
C     Local variables
C
      INTEGER        J
      INTEGER        M
      INTEGER        P
      INTEGER        Q
      INTEGER        I
      INTEGER        B
      INTEGER        Z

C
C     SPICELIB functions
C
      LOGICAL               RETURN

C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'STF2I2' )
      END IF

C
C     Set initial output value to zero. Should the error handling be
C     set to RETURN and this routine fail, zero will be returned.
C
      STF2I2 = 0

C
C     The number of integers to be packed must be positive and whether 
C     less than or equal to 16 (parameterized in NIIMAX.)
C
      IF ( N .LE. 0 ) THEN
         CALL SETMSG ( 'The number of integers to be packed must ' //
     .                 'be greater than zero. It was #.'           )
         CALL ERRINT ( '#', N                                      )
         CALL SIGERR ( 'SPICE(TOOFEWINTEGERS)'                     )
         CALL CHKOUT ( 'STF2I2'                                    ) 
         RETURN
      END IF

      IF ( N .GT. NIIMAX ) THEN
         CALL SETMSG ( 'The number of integers to be packed must ' //
     .                 'be less than #. It was #.'                 )
         CALL ERRINT ( '#', N                                      )
         CALL ERRINT ( '#', NIIMAX                                 )
         CALL SIGERR ( 'SPICE(TOOMANYINTEGERS)'                    )
         CALL CHKOUT ( 'STF2I2'                                    )
         RETURN
      END IF

C
C     If we need to pack one integer into one integer, we simply 
C     return what we have on the input.
C
      IF ( N .EQ. 1 ) THEN
         STF2I2 = A(1)
         CALL CHKOUT ( 'STF2I2' )
         RETURN
      END IF

C
C     First, initialize auxiliary variables.
C
      J = 0
      M = 32 / N
      P = 2**M
      Q = P/2

C
C     Second, verify whether all the integers to be packed can be 
C     represented by the number of bits that will be used for them.
C
      DO I = 1, N

         IF ( IABS( A(I) ) .GE. Q ) THEN
            CALL SETMSG ( 'The integer number #, which is the # '  //
     .                    'element of the input array, cannot be ' //
     .                    'represented by # bits dedicated to it.' )
            CALL ERRINT ( '#', A(I)                                )
            CALL ERRINT ( '#', I                                   )
            CALL ERRINT ( '#', M                                   )
            CALL SIGERR ( 'SPICE(VALUEISTOOLARGE)'                 )
            CALL CHKOUT ( 'STF2I2'                                 ) 
            RETURN
         END IF

      END DO

C
C     Everything is ready -- let's pack the numbers!
C
      DO I = 0, N-1

C
C        B is the next number to be packed into J (the Nth element of
C        A is packed first, N-1th -- second, etc.) Z determines at what
C        bit range this number is going to be packed into.
C
         B = A(N-I)
         Z = 2**(M*I)

         IF ( I .LT. N-1 ) THEN

C
C           We pack all numbers but the first using the same approach:
C           if the number is negative, we turn into a positive number
C           with the same bit pattern.
C
            IF ( B .LT. 0 ) THEN
               B = B + P
            END IF

C
C           and then multiply it by Z (which determines its location in
C           the output 32 bits) and add it to the output number.
C
            J = J + B * Z

         ELSE 

C
C           The first number of array requires special attention.
C           
            IF ( B .GE. 0 ) THEN

C
C              If it's greater than or equal to zero and, therefore,
C              it's addition is NOT going to affect sign bit, it's
C              simply multiplied by Z and added to the output value.
C
               J = J + B * Z

            ELSE 

               IF ( MOD(32,N) .EQ. 0 ) THEN

C
C                 If it's less than zero and may be using the sign 
C                 bit -- 32th bit, -- we subtract the value 
C                 accumulated up till this point from it. 
C
                  J = B * Z - J

               ELSE

C
C                 If it's less than zero and but is NOT using the sign 
C                 bit -- 32th bit, -- we turn into a positive number 
C                 with the same bit pattern and add it to the output 
C                 value.
C
                  J = J + ( B + P ) * Z

               END IF

            END IF

         END IF

      END DO

C
C     All done -- assign output value and check out.
C
      STF2I2 = J

      CALL CHKOUT ( 'STF2I2' )

      RETURN

      END
      
