C$Procedure      STFATT ( get attributes for all sky catalog fields )

      SUBROUTINE STFATT ( NFLDS, FNAMES, FTYPES, FCLASS, FINDXD )

C$ Abstract
C
C     Fetch generic attributes of the fields in the loaded SPICE Sky 
C     catalog.
C
C$ Copyright
C
C     Copyright (2000), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     SPICE Sky Catalog Subsystem
C
C$ Keywords
C
C     TBD
C
C$ Declarations

      INTEGER               NFLDS
      CHARACTER*(*)         FNAMES (*)
      CHARACTER*(1)         FTYPES (*)
      CHARACTER*(*)         FCLASS (*)
      LOGICAL               FINDXD (*)

C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     NFLDS      O   The number of fields in the catalog
C     FNAMES     O   Field names
C     FTYPES     O   Field types
C     FCLASS     O   Field classes
C     FINDXD     O   Field indexing flags
C
C$ Detailed_Input
C
C     None
C
C$ Detailed_Output
C
C     NFLDS          is the number of fields in the loaded SPICE Sky
C                    catalog.
C
C     FNAMES         is the array of the field names.
C
C     FTYPES         is the array of the field types: 'C' - text field,
C                    'I' - integer field, 'D' - double precision field.
C
C     FCLASS         is the array of the field classes, namely: 'ID',
C                    'RA', 'DEC', 'DEC_BAND', 'RA_ZONE', 'RA_SHIFT',
C                    'DEC_SHIFT', 'RA_PM', 'DEC_PM', 'PARALLAX', 
C                    'RADIAL_VELOCITY', 'INTENSITY', 'CLASS_INDEX', 
C                    'OBJECT_TYPE'.
C
C     FINDXD         is the array of the field indexing flag: .TRUE. if
C                    field is indexed, .FALSE. if field is not indexed.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) If no sky catalog was loaded before calling this routine,
C        then the error 'SPICE(NOLOADEDCATALOG)' will be signalled.
C
C     2) If loaded catalog type is not supported by this routine,
C        then the error 'SPICE(UNSUPPORTEDTYPE)' will be signalled.
C
C$ Files
C
C     TBD
C
C$ Particulars
C
C     TBD -- needs to be re-written.
C
C     The routine provides user with review of fields
C     characteristics, namely the number of fields, field names,
C     field types and classes. We shall distinguish two kinds of
C     field types: numerical and text. The following field classes
C     are introduced: 'ID', 'RA', 'DEC', 'DEC_BAND', 'RA_ZONE', 
C     'RA_SHIFT', 'DEC_SHIFT', 'RA_PM', 'DEC_PM', 'PARALLAX', 
C     'RADIAL_VELOCITY', 'INTENSITY', 'CLASS_INDEX', 'OBJECT_TYPE', 
C     but only 'ID' is to exist in any case. 'INTENSITY' class is 
C     expected to be specified.
C
C$ Examples
C
C     C
C     C     This program demonstrates high-level "summary" routines 
C     C     of the SPICE Sky Catalog System. It displays 
C     C     meta-information for a SPICE Sky catalog, the name of 
C     C     which was provided in response to the prompt.
C     C
C           PROGRAM               SUMMRY
C     
C           IMPLICIT              NONE
C     
C     C
C     C     Sky catalog  parameters
C     C
C           INCLUDE               'starlib.inc'
C     
C     C
C     C     Local parameters.
C     C
C           INTEGER               NDESCR
C           PARAMETER           ( NDESCR = 100 )
C     
C     C
C     C     Local variables.
C     C
C           CHARACTER*(1)         FTYPE  (NFLMAX)
C           CHARACTER*(CATNLN)    CATNAM
C           CHARACTER*(DSCLSZ)    CATFIL
C           CHARACTER*(DSCLSZ)    DESCR  (NDESCR)
C           CHARACTER*(DSCLSZ)    LSKFIL
C           CHARACTER*(DSCLSZ)    STRING
C           CHARACTER*(FLDCLN)    FCLASS (NFLMAX)
C           CHARACTER*(FLDNLN)    FNAME  (NFLMAX)
C           CHARACTER*(FRMNLN)    FRAME
C           CHARACTER*(OBJTLN)    OBTYPE
C     
C           DOUBLE PRECISION      EPOCH
C           DOUBLE PRECISION      MATRIX (3,3)
C     
C           INTEGER               FIRST
C           INTEGER               I
C           INTEGER               IFLD
C           INTEGER               J
C           INTEGER               N
C           INTEGER               NFLDS
C           INTEGER               NOBJ
C     
C           LOGICAL               FINDX  (NFLMAX)
C     
C     C
C     C     Prompt for catalog file and LSK file names.
C     C
C           CALL TOSTDO ( ' '                                          )
C           CALL PROMPT ( 'Enter Catalog File Name > ', CATFIL         )
C           CALL TOSTDO ( ' '                                          )
C           CALL PROMPT ( 'Enter LSK File name > ', LSKFIL             )
C           CALL TOSTDO ( ' '                                          )
C     
C     C
C     C     Load catalog and LSK files.
C     C
C           CALL FURNSH ( CATFIL )
C           CALL STLOAD ( CATFIL )
C           CALL FURNSH ( LSKFIL )
C     
C     C
C     C     Obtain catalog meta-information.
C     C
C           CALL STCATT ( CATNAM, OBTYPE, EPOCH, FRAME, MATRIX, NOBJ )
C     
C     C
C     C     Display summary header. (TOSTDO is a SPICELIB routine. It 
C     C     print a string to STDOUT and makes sure that the string 
C     C     appearance on on the terminal in identical on all computer 
C     C     platforms.)
C     C
C           CALL TOSTDO ( '****************************************' //
C          .              '****************************************'   ) 
C           CALL TOSTDO ( ' '                                          )
C           CALL TOSTDO ( 'Summary for SCK file: ' // CATFIL           )
C           CALL TOSTDO ( 'Leapseconds File    : ' // LSKFIL           )
C           CALL TOSTDO ( ' '                                          ) 
C           CALL TOSTDO ( '----------------------------------------' //
C          .              '----------------------------------------'   )
C     
C     C
C     C     Display catalog meta-information.
C     C
C           CALL TOSTDO ( ' '                                          ) 
C           CALL TOSTDO ( 'Catalog Meta-information'                   )
C           CALL TOSTDO ( ' '                                          )
C           CALL TOSTDO ( '----------------------------------------' //
C          .              '----------------------------------------'   )
C     
C           CALL TOSTDO ( ' '                                          ) 
C           CALL TOSTDO ( 'Catalog Name        : ' // CATNAM           )
C           
C           CALL INTSTR ( NOBJ, STRING )
C           CALL TOSTDO ( 'Number of Objects   : ' // STRING           )
C     
C           CALL TOSTDO ( 'Catalog Object Type : ' // OBTYPE           )
C     
C           CALL TIMOUT ( EPOCH, 'YYYY-MM-DDTHR:MN:SC.###', STRING )
C           CALL TOSTDO ( 'Catalog EPOCH (UTC) : ' // STRING           )
C     
C           CALL TOSTDO ( 'Catalog Frame       : ' // FRAME            )
C     
C           CALL TOSTDO ( ' '                                          )
C           CALL TOSTDO ( 'Catalog Frame to J2000 Rotation Matrix '    )
C           CALL TOSTDO ( '-------------------------------------- '    )
C     
C           DO I = 1, 3
C              STRING = '   #   #   #'
C              DO J = 1, 3
C                 CALL REPMD  ( STRING, '#', MATRIX(I,J), 16, STRING )
C              END DO
C              CALL TOSTDO ( STRING                                    )
C           END DO
C     
C     C
C     C     Display catalog description.
C     C
C           CALL TOSTDO ( ' '                                          )
C           CALL TOSTDO ( 'Catalog description'                        )
C           CALL TOSTDO ( '-------------------'                        )
C     
C           N     = 1
C           FIRST = 1
C           DO WHILE ( N.GT.0 )
C     
C     C
C     C        Get next set of lines from catalog description.
C     C
C              CALL STCDSC ( NDESCR, FIRST, N, DESCR )
C     
C              DO I = 1, N
C                 CALL TOSTDO ( DESCR(I)                               ) 
C              END DO
C     
C              FIRST = FIRST + N
C     
C           END DO
C     
C     C
C     C     Obtain meta-information for the catalog fields.
C     C
C           CALL STFATT ( NFLDS, FNAME, FTYPE, FCLASS, FINDX )
C     
C     C
C     C     Display catalog fields meta-information.
C     C
C           CALL TOSTDO ( ' '                                          ) 
C           CALL TOSTDO ( '----------------------------------------' //
C          .              '----------------------------------------'   )
C           CALL TOSTDO ( ' '                                          ) 
C           CALL TOSTDO ( 'Catalog Fields Meta-information'            )
C           CALL TOSTDO ( ' '                                          )
C           CALL TOSTDO ( '----------------------------------------' //
C          .              '----------------------------------------'   )
C     
C           CALL TOSTDO ( ' '                                          )
C           CALL INTSTR ( NFLDS, STRING )
C           CALL TOSTDO ( 'Number of Fields    : ' // STRING           )
C     
C     C
C     C     Display information for each field.
C     C
C           DO IFLD = 1, NFLDS
C     
C              CALL TOSTDO ( ' '                                       )
C              CALL TOSTDO ( '----------------------------------------'//
C          .                 '----------------------------------------')
C              CALL TOSTDO ( ' '                                       )
C     
C              CALL INTSTR ( IFLD, STRING )
C              CALL TOSTDO ( 'Field Number        : ' // STRING        )
C     
C              CALL TOSTDO ( 'Field Name          : ' // FNAME(IFLD)   )
C     
C              IF      ( FTYPE(IFLD) .EQ. 'C' ) THEN
C                 CALL TOSTDO ( 'Field Type          : CHARACTER'      )
C              ELSE IF ( FTYPE(IFLD) .EQ. 'D' ) THEN
C                 CALL TOSTDO ( 'Field Type          : DOUBLE PRECISION')
C              ELSE IF ( FTYPE(IFLD) .EQ. 'I' ) THEN
C                 CALL TOSTDO ( 'Field Type          : INTEGER'        )
C              END IF
C     
C              IF ( FCLASS(IFLD) .EQ. ' ' ) THEN
C                 CALL TOSTDO ( 'Field Class         : <not specified>')
C              ELSE
C                 CALL TOSTDO ( 'Field Class         : ' //FCLASS(IFLD))
C              END IF
C     
C              IF ( FINDX(IFLD) ) THEN
C                 CALL TOSTDO ( 'Field Indexing Flag : .TRUE.'         )
C              ELSE
C                 CALL TOSTDO ( 'Field Indexing Flag : .FALSE.'        ) 
C              END IF
C              CALL TOSTDO ( 'Field Description'                       )
C              CALL TOSTDO ( '-----------------'                       )
C     
C              N     = 1
C              FIRST = 1
C              DO WHILE ( N .GT. 0 )
C     
C     C
C     C           Obtain field description.
C     C
C                 CALL STFDSC ( FNAME (IFLD), NDESCR, FIRST, N, DESCR )
C     
C                 DO I = 1, N
C                    CALL TOSTDO ( DESCR(I) )
C                 END DO
C     
C                 FIRST = FIRST + N
C     
C              END DO
C     
C           END DO
C     
C           CALL TOSTDO ( ' '                                          )
C           CALL TOSTDO ( '****************************************' //
C          .              '****************************************'   ) 
C           CALL TOSTDO ( ' '                                          )
C     
C     C
C     C     All done: unload the catalog ...
C     C
C     C      CALL UNLOAD ( CATFIL )
C           CALL STUNLD ( )
C     
C           END
C     
C$ Restrictions
C
C     SPICE Sky catalog file must be loaded using STLOAD routine 
C     prior to calling this routine.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     V.N. Kheifets  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 01-DEC-1999 (VNK)
C
C-&

C$ Index_Entries
C
C     get generic sky_catalog field attributes
C
C-&

C
C     SPICELIB functions
C
      LOGICAL               RETURN

C
C     Local variables
C
      INTEGER               TYPE

C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'STFATT' )
      END IF

C
C     Get the catalog type and call corresponding type-specific 
C     routine.
C
      CALL STTYPE ( TYPE )

      IF ( TYPE .EQ. 0 ) THEN

         CALL SETMSG ( 'Must load catalog before setting constraints.' )
         CALL SIGERR ( 'SPICE(NOLOADEDCATALOG)'                        )

      ELSE IF ( TYPE .EQ. 2 ) THEN

         CALL STFIN2 ( NFLDS, FNAMES, FTYPES, FCLASS, FINDXD )

      ELSE

         CALL SETMSG ( 'Cannot work with catalog type #.' )
         CALL ERRINT ( '#', TYPE                          )
         CALL SIGERR ( 'SPICE(UNSUPPORTEDTYPE)'           )

      END IF

      CALL CHKOUT ( 'STFATT' )
      
      RETURN
      
      END
