C$Procedure      STFCH2 ( get objects met FOV/constraints limitations )

      SUBROUTINE STFCH2 ( MAX, NRET, LIST )

C$ Abstract
C
C     Returns numbers of the sky catalog objects from the loaded
C     SPICE Type 2 Sky Catalog that satisfy all search constraints
C     specified earlier using STSCN2, STSCC2, STSEP2 and STSF2
C     routines.
C
C$ Copyright
C
C     Copyright (2000), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     SPICE Sky Catalog Subsystem
C
C$ Keywords
C
C     TBD
C
C$ Declarations

      IMPLICIT              NONE

      INCLUDE               'starlib.inc'

      INTEGER               MAX
      INTEGER               NRET
      INTEGER               LIST ( * )

C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     MAX        I   maximal amount of record numbers to be returned
C     NRET       O   the number of returned elements
C     LIST       O   record numbers for objects met the constraints
C
C$ Detailed_Input
C
C     MAX            is maximal amount of record numbers that can be
C                    accommodated in the array LIST.
C
C$ Detailed_Output
C
C     NRET           is the number of elements returned in LIST.
C
C                    If NRET equal to MAX, than possibly more record
C                    numbers can be returned by additional calls to
C                    this routine.
C
C                    If NRET equal 0, then to sky objects meet all
C                    search constraints or all matching record numbers
C                    were fetched by previous calls to this routine.
C
C     LIST           is the array containing all or a part of all
C                    record numbers of sky catalog records that
C                    satisfy all sky catalog search constraints.
C
C$ Parameters
C
C     See include file.
C
C$ Files
C
C     TBD 
C
C$ Exceptions
C
C     1) If no type Type 2 catalog is loaded at the time when this
C        routine is called, then the error 'SPICE(CATALOGISNOTLOADED)'
C        will be signalled.
C
C     2) If room specified by input argument MAX is not a positive
C        number, then the error 'SPICE(BADROOMVALUE)' will be
C        signalled.
C
C     3) more to come ...
C
C$ Particulars
C
C     TBD -- needs to be re-written
C
C$ Examples
C
C     TBD -- needs partial example.
C
C$ Restrictions
C
C     Type 2 SPICE Sky catalog must be loaded using STLD2 routine
C     prior to calling this routine.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     V.N. Kheifets  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 01-JUL-1999 (VNK)
C
C-&

C$ Index_Entries
C
C     TBD
C
C-&

C
C     Local variables
C
      INTEGER               NOBJ
      INTEGER               NCONST
      INTEGER               NINDXD
      INTEGER               INDXED (2,NFLMAX)
      INTEGER               NGOOD
      INTEGER               IREC
      INTEGER               ICONST
      INTEGER               IBAND
      INTEGER               IBAND1
      INTEGER               IBAND2
      INTEGER               IZONE
      INTEGER               IZONE1
      INTEGER               IZONE2
      INTEGER               RECNO1
      INTEGER               RECNO2
      INTEGER               I
      INTEGER               IBEG
      INTEGER               IEND
      INTEGER               I1
      INTEGER               I2
      INTEGER               N
      INTEGER               NMIN
      INTEGER               HANDLE
      INTEGER               LIST1  (NOBMAX)

      LOGICAL               FOVSET
      LOGICAL               OK
      LOGICAL               FIRST
      LOGICAL               SRCHST

      CHARACTER*(FRMNLN)    CFRAME

      DOUBLE PRECISION      RA
      DOUBLE PRECISION      DEC
      DOUBLE PRECISION      R       (3)

      LOGICAL               RETURN

      SAVE

      DATA                  FIRST  / .TRUE. /

C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'STFCH2' )
      END IF

C
C     Check whether input space is positive non-zero number.
C
      IF ( MAX .LE. 0 ) THEN
         CALL SETMSG ( 'Room specified by input argument MAX was '   //
     .                 'not a positive number. It was #.'            )
         CALL ERRINT ( '#', MAX                                      )
         CALL SIGERR ( 'SPICE(BADROOMVALUE)'                         )
         CALL CHKOUT ( 'STFCH2'                                      )
         RETURN
      END IF

C
C     Check whether the catalog is really loaded.
C
      CALL STCHL2 ( OK )
      IF ( .NOT. OK ) THEN
         CALL SETMSG ( 'No Type 2 Sky catalog was loaded prior '     //
     .                 'calling this routine.'                       )
         CALL SIGERR ( 'SPICE(CATALOGISNOTLOADED)'                   )
         CALL CHKOUT ( 'STFCH2'                                      )
         RETURN
      END IF

C
C     Check whether catalog that we use or any of the constraints
C     changed in the middle of current fetch loop.
C
      CALL STSSS2( 'GET', SRCHST )

      IF ( .NOT. SRCHST ) THEN

C
C        Yes, something did -- we have to do full-scale search again.
C
         FIRST = .TRUE.

      END IF

C
C     Was fetch called the first time in a series?
C
      IF ( FIRST ) THEN

C
C        Get constraint information.
C
         CALL STINF2 ( FOVSET, NCONST, NINDXD, INDXED,
     .                           NOBJ, CFRAME, HANDLE )

C
C        Initialize number of matches and index of the first element
C        to be returned.
C
         NGOOD = 0
         I1    = 1

         IF ( NCONST .EQ. -1 ) THEN

C
C           First special case -- no objects can be returned because
C           a brave user defined constraint that is outside of the
C           contrained field range.
C

         ELSE IF ( .NOT. FOVSET .AND. NCONST .EQ. 0 ) THEN

C
C           First special case -- user wants all objects. So be it.
C
            NGOOD = NOBJ

            DO I = 1, NGOOD

C
C              WARNING! At this point we cannot buffer more than
C              NOBMAX objects. Someday we will grow up and figure
C              out how to fix it, but now we signal an error.
C
               IF ( I .GT. NOBMAX ) THEN
                  CALL SETMSG ( 'Cannot buffer more than # records.' )
                  CALL SIGERR ( 'SPICE(TOBEFIXEDLATER1)'             )
                  CALL CHKOUT ( 'STFCH2'                             )
                  RETURN
               END IF

               LIST1(I)=I

            END DO

         ELSE IF ( .NOT. FOVSET .AND. NINDXD .EQ. 0 ) THEN

C
C           Second special case -- user set some constraints without
C           setting FOV, but none of the constrained field is indexed.
C           So we will have to read EVERY single record anyway, so why
C           don't do it in this IF pocket (this may potentially save a
C           hassle to deal with too many records.)
C
            DO IREC = 1, NOBJ

C
C              Buffer record and check it for all constraints.
C
               CALL STREC2  ( IREC  )
               CALL STCHC2  ( OK )
****           print*,'STFCH2: checked record',irec,'  OK =   ',OK

               IF ( OK ) THEN

                  NGOOD = NGOOD + 1

C
C                 WARNING! At this point we cannot buffer more than
C                 NOBMAX objects. Someday we will grow up and figure
C                 out how to fix it, but now we signal an error.
C
                  IF ( NGOOD .GT. NOBMAX ) THEN
                     CALL SETMSG ( 'Cannot buffer more than # records.')
                     CALL SIGERR ( 'SPICE(TOBEFIXEDLATER2)'            )
                     CALL CHKOUT ( 'STFCH2'                            )
                     RETURN
                  END IF

                  LIST1(NGOOD) = IREC

               END IF

            END DO

         ELSE

C
C           Check whether there are any indexed constraints
C
            IF ( NINDXD .GT. 0 ) THEN

C
C              There are indexed constraints
C
               NMIN = NOBMAX

               DO I = 1, NINDXD

C
C                 Carry out fetching of consequent Index Tables subsets
C                 and looking for one that has smallest number of
C                 elements.
C
                  CALL STGL2 ( INDXED(1,I), INDXED(2,I), I1, I2 )

                  IF ( I2 - I1 + 1 .LT. NMIN ) THEN
                     NMIN = I2 - I1 + 1
                     ICONST = I
                     IBEG = I1
                     IEND = I2
                  END IF

               END DO

C
C              Now we are sure that the minimal subset corresponds
C              to constraint # ICONST and consists of NMIN elements
C              with record numbers stored in our DAS file from IBEG
C              to IEND.
C

            END IF

C
C           Check whether FOV is set.
C

C
C           First, initialize counter.
C
            N = 0

            IF ( FOVSET ) THEN

C
C              Fill LIST1 using only geometric indexing info.
C
               CALL STBOX2
               CALL STGB2 ( IBAND1, IBAND2 )
               DO IBAND = IBAND1, IBAND2

                  CALL STGZ2 ( IBAND, IZONE1, IZONE2 )

                  DO IZONE = IZONE1, IZONE2

                     CALL STGR2 ( IBAND, IZONE, RECNO1, RECNO2 )

                     DO IREC = RECNO1, RECNO2

                        N = N + 1
C
C                       WARNING! At this point we cannot buffer more
C                       than NOBMAX objects. Someday we will grow up
C                       and figure out how to fix it, but now we signal
C                       an error.
C
                        IF ( N .GT. NOBMAX ) THEN
                           CALL SETMSG ( 'Cannot buffer more than #.' )
                           CALL SIGERR ( 'SPICE(TOBEFIXEDLATER3)'     )
                           CALL CHKOUT ( 'STFCH2'                     )
                           RETURN
                        END IF

                        LIST1(N) = IREC

                     END DO

                  END DO

               END DO

            END IF

C
C           Now let us compare size of the list generated by FOV
C           restrictions and obtained subset of indexed constraints
C           (if each of them exists)
C
            IF ( NINDXD .GT. 0 .AND.
     .         ( .NOT. FOVSET .OR. N .GT. NMIN ) ) THEN

              CALL DASRDI ( HANDLE, IBEG, IEND, LIST1 )
              N = NMIN

            END IF

C
C           And the final filtering loop will be only for the records
C           that are still left in LIST1. Here we will use field data
C           to check against all constraints and check position
C           adjusted for PM against FOV.
C
            DO I = 1, N

               IREC = LIST1 ( I )

               CALL STREC2  ( IREC )

C
C              Check record for all constraints.
C

               CALL STCHC2  ( OK )
****           print*,'STFCH2: checked record',irec,'  OK =   ',OK

               IF ( OK ) THEN

C
C                 If FOV was set, also check proper motion adjusted
C                 RA/DEC against FOV.
C
                  IF ( FOVSET ) THEN

C
C                    Generate unit vector R for the object according
C                    to its RA and DEC for the Catalog epoch and in
C                    the Catalog frame.
C
                     CALL STGRD2 ( IREC, CFRAME, RA, DEC )
                     CALL RADREC ( 1D0, RA, DEC, R )

C
C                    Check whether the unit vector R is within
C                    pre-defined FOV.
C
                     CALL STCHF2 ( R, OK )

                  END IF

C
C                 If still OK, this is one of the objects we want!
C
                  IF ( OK ) THEN

                     NGOOD = NGOOD + 1
C
C                    WARNING! At this point we cannot buffer more than
C                    NOBMAX objects. Someday we will grow up and figure
C                    out how to fix it, but now we signal an error.
C
                     IF ( NGOOD .GT. NOBMAX ) THEN
                        CALL SETMSG ( 'Cannot buffer more than # rec.')
                        CALL SIGERR ( 'SPICE(TOBEFIXEDLATER6)'        )
                        CALL CHKOUT ( 'STFCH2'                        )
                        RETURN
                     END IF

                     LIST1(NGOOD) = IREC

                  END IF

               END IF

            END DO

         END IF

C
C        Initialize the first element of LIST1 array and close the door
C
         I1 = 1
         FIRST = .FALSE.

C
C        If also need to tell the  reader umbrella that fetch is
C        in progress (to make sure that it tells us back if someone
C        screwed around with constraints during the fetch :-)
C
         SRCHST = .TRUE.
         CALL STSSS2( 'SET', SRCHST )

      END IF

C
C     Now it's high time to return all (but not more than MAX) fetched
C     record numbers. Copy returned elements to LIST array and modify
C     the number of "good" elements and index for next fetch.
C
      NRET = MIN0 ( NGOOD, MAX )

      DO I = 1, NRET
         LIST(I) = LIST1(I+I1-1)
      END DO
      NGOOD = NGOOD - NRET
      I1 = I1 + NRET

C
C     If we have returned everything -- assuming that the caller
C     fetched portions of the buffer sequentially till he got all
C     records -- we can re-set first pass and fetch state statuses.
C
      IF ( NRET .EQ. 0 ) THEN

         FIRST  = .TRUE.

         SRCHST = .FALSE.
         CALL STSSS2( 'SET', SRCHST )

      END IF

      CALL CHKOUT ( 'STFCH2' )
      RETURN

      END
		
