C$Procedure      STGIT2 ( Get address range for metaitem, Sky Catalog 2)

      SUBROUTINE STGIT2 ( HANDLE, NAME, IBEGIN, IEND )

C$ Abstract
C
C     Get the DAS integer address range of the certain item (array)
C     in the Type 2 Sky catalog metainformation block.
C
C$ Copyright
C
C     Copyright (2000), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     SPICE Sky Catalog Subsystem
C
C$ Keywords
C
C     TBD
C
C$ Declarations

      IMPLICIT              NONE

      INCLUDE               'starlib.inc'

      INTEGER               HANDLE
      CHARACTER*(*)         NAME
      INTEGER               IBEGIN
      INTEGER               IEND

C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     HANDLE     I   Handle of the DAS file
C     NAME       I   Name of the meta array to be found
C     IBEGIN     O   Initial address of the array
C     IEND       O   Final address of the array
C
C$ Detailed_Input
C
C     HANDLE         is the handle of an opened DAS file.
C
C     NAME           is the name of the metainformation array to be
C                    located in the DAS file.
C
C$ Detailed_Output
C
C     IBEGIN         is initial address of the array.
C
C     IEND           is final address of the array.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) If array with specified name was not found in the meta
C        directory of the Sky catalog file, then the error
C        'SPICE(BADMETAARRYNAME)' will be signalled.
C
C     2) If DAS reader routine called by this routine will fails,
C        then it signals appropriate error.
C
C$ Files
C
C     This routine operates on Type 2 SPICE Sky Catalog file attached
C     to HANDLE and opened for read access.
C
C$ Particulars
C
C     This routine returns the DAS integer address range of the certain
C     item (array) in the Type 2 Sky catalog metainformation block.
C
C     At the first BEGMET-1 characters of the file are are reserved for
C     META DIRECTORY. It is intended to make it easy to augment the
C     METAINFORMATION segment with additional information during
C     modifications, to fetch the information from the segment and to
C     support fast and easy access to the guide information of the
C     catalog. We shall consider the information as a set of integer
C     arrays (scalars could be considered as arrays of 1 element). So,
C     META DIRECTORY structure is as follows:
C
C        -- N - the number of filled directory elements (one integer)
C
C        -- Meta information Item 1 (six integers)
C
C        -- Meta information Item 2 (six integers)
C
C        --   ...
C
C        -- Meta information Item N (six integers)
C
C     This diagram illustrates META DIRECTORY and METAINFORMATION 
C     block layout (each square represents one integer number; DAS 
C     address of the number is given in the top left corner of the 
C     square): 
C        
C              +-------+    N contains address of the last used
C       META   |1      |        element of the META DIRECTORY
C     DIRECTORY|   N   | <-----------------------------------------.
C              |       |                                           |
C              +-------+-------+-------+-------+-------+-------+   |
C              |2                              |6 Beg. |7 End  |   |
C              |     Name of the Array # 1     | Addr 1| Addr 1|   |
C              |                               | (BA1) | (EA1) |   |
C              +-------+-------+-------+-------+-------+-------+   |
C              |8                              |12 Beg.|13 End |   |
C              |     Name of the Array # 2     | Addr 2| Addr 2|   |
C              |                               | (BA2) | (EA2) |   |
C              +-------+-------+-------+-------+-------+-------+   |
C              |14                             |18     |19     |   |
C                 ..........................................       |
C              |                               |       |       |   |
C              +-------+-------+-------+-------+-------+-------+   |
C              |(N-1)*6+2                      |  Beg  |  End  |   |
C              |     Name of the Array # n     | Addr n| Addr n| <-'
C              |                               | (BAn) | (EAn) |   
C              +-------+-------+-------+-------+-------+-------+
C              |                               |       |       |   
C                 ..........................................   
C              |                               |       |       |   
C              +-------+-------+-------+-------+-------+-------+
C     METAINFO |BA1                     EA1    |BA2            |   
C     starts @ |      Array # 1 data           |               |   
C      BEGMET  |                               |               |   
C              +-------+-------+-------+-------+-------+-------+
C              |                                               |   
C              |                Array # 2 Data                 |   
C              |                                               |   
C              +-------+-------+-------+-------+-------+-------+
C              |         EA2   |BA3/EA3|BA4                    |   
C              |               |       |     Array # 4 Data    |   
C              |               |       |                       |   
C              +-------+-------+-------+-------+-------+-------+
C              |                                               |   
C                 ..........................................
C              |                                               |   
C              +-------+-------+-------+-------+-------+-------+
C              |        EA(n-1)|BAn             EAn    |BA(n+1)|   
C              |               |   Array # n Data      |       |   
C              |               |                       |       |   
C              +-------+-------+-------+-------+-------+-------+
C              |                                               |   
C                 ..........................................
C
C     Each of the metainformation items is presented by six integers.
C     The first four of them present name of the item in coded form,
C     the 5th and the 6th integer are the catalog file addresses 
C     corresponding to the first and to the last elements of the array.
C
C$ Examples
C
C     In this example STGIT2 is called to obtain address ranges for 
C     the catalog meta information item (array) containing catalog 
C     name (the name of this metainformation item is parameterized 
C     in MDCNAM in the include file):
C
C           INCLUDE               'starlib.inc'
C
C           CHARACTER*(DSCLSZ)    FILNAM
C           CHARACTER*(CATNLN)    CATNAM
C
C           INTEGER               HANDLE
C           INTEGER               I
C           INTEGER               J
C           INTEGER               Z ( CATNLN / NCHARI )
C
C     C
C     C     Prompt for and open for read DAS file containing 
C     C     Type 2 Sky catalog for read.
C     C
C           CALL PROMPT ( 'Enter catalog file name >', FILNAM )       
C           CALL DASOPR ( FILNAM, HANDLE )
C
C     C
C     C     Get address range at which packed catalog name is stored 
C     C     in the metainformation area of the file.
C     C
C           CALL STGIT2 ( HANDLE, MDCNAM, I, J )
C
C     C
C     C     Read integers representing catalog name and unpack it
C     C     from them.
C     C
C           CALL DASRDI ( HANDLE, I, J, Z  )
C           CALL STI2T2 ( Z, J - I + 1, CATNAM, J )
C 
C     C
C     C     Print out catalog name.
C     C 
C           WRITE (*,*) 'Catalog name: ', CATNAM
C
C     C
C     C     Close catalog file.
C     C
C           CALL DASCLS ( HANDLE )
C
C           END
C
C$ Restrictions
C
C     Catalog file attached to HANDLE must be opened for read access 
C     prior to calling this routine.
C
C     Requested metainformation item (array) must be present in the 
C     catalog file attached to HANDLE.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     V.N. Kheifets   ( JPL )
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 1-DECEMBER-1999 (VNK)
C
C-&

C$ Index_Entries
C
C     read meta_array address range from type_2 sky catalog file
C
C-&

C
C     SPICELIB functions
C
      LOGICAL               RETURN

C
C     Local variables
C
      CHARACTER*(METNLN)    NAM

      INTEGER               ARRAY ( 6 )
      INTEGER               I
      INTEGER               J
      INTEGER               K
      INTEGER               NROWS

C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'STGIT2' )
      END IF

C
C     Read the total number of integers that make up the Type 2
C     catalog meta information block directory (including the number
C     that contains K itself.)
C
      CALL DASRDI ( HANDLE, 1, 1, K )

C
C     Find out how many meta information items (arrays) we have.
C
      NROWS = ( K - 1 ) / 6

      J = 2

      DO I = 1, NROWS

C
C        Read encoded name (4 numbers) and start and stop addresses for
C        the next array.
C
         CALL DASRDI ( HANDLE, J, J + 5, ARRAY )

C
C        Decode the first METNLN/NCHARI integers to meta array character 
C        name.
C
         CALL STI2T2 ( ARRAY, METNLN/NCHARI, NAM, K )

C
C        Since all array names are parameterized, we can check for
C        exact equality.
C
         IF ( NAM .EQ. NAME ) THEN

C
C           We found it -- re-assign addressed and return.
C
            IBEGIN = ARRAY(5)
            IEND   = ARRAY(6)

            CALL CHKOUT ( 'STGIT2' )

            RETURN

         END IF

         J = J + 6

      END DO

C
C     If we are here, we failed to find given name in META DIRECTORY.
C
      CALL SETMSG ( 'Could not find meta array with name ''#'' in ' //
     .              'the META DIRECTORY of the Sky catalog file.'   )
      CALL ERRCH  ( '#', NAME                                       )
      CALL SIGERR ( 'SPICE(BADMETAARRYNAME)'                        )
      CALL CHKOUT ( 'STGIT2'                                        )

      RETURN

      END
