C$Procedure      STGRD ( Get sky object position -- RA and DEC )

      SUBROUTINE STGRD ( RECNO, FRAME, RA, DEC )

C$ Abstract
C
C     Returns position -- RA and DEC -- adjusted for proper motion in
C     a specified frame for sky catalog object with a given catalog
C     record number.
C
C$ Copyright
C
C     Copyright (2000), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     SPICE Sky Catalog Subsystem
C
C$ Keywords
C
C     TBD
C
C$ Declarations

      IMPLICIT              NONE

      INTEGER               RECNO
      CHARACTER*(*)         FRAME
      DOUBLE PRECISION      RA
      DOUBLE PRECISION      DEC

C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     RECNO      I   SPICE sky catalog record number
C     FRAME      I   Reference frame in which position is returned
C     RA         O   Right ascension of the sky object
C     DEC        O   Declination of the sky object
C
C$ Detailed_Input
C
C     RECNO          is the SPICE catalog record number.
C
C     FRAME          is the name of the frame in which the position
C                    must be returned.
C
C$ Detailed_Output
C
C     RA             is right ascension of the object in the frame FRAME
C                    adjusted for the epoch specified by call(s) for
C                    STSEPH/STSFOV.
C
C     DEC            is declination of the object in the frame FRAME
C                    adjusted for the epoch specified by call(s) for
C                    STSEPH/STSFOV.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) If no sky catalog was loaded before calling this routine,
C        then the error 'SPICE(CATALOGISNOTLOADED)' will be signalled.
C
C     2) If loaded catalog type is not supported by this routine,
C        then the error 'SPICE(UNSUPPORTEDTYPE)' will be signalled.
C
C$ Files
C
C     TBD
C
C$ Particulars
C
C     TBD -- needs more details, especially about epoch/PM/etc.
C
C     The routine fetches RA and DEC double precision values
C     (in radians) for the sky object with the pre-defined record
C     number and recalculates the position to another EPOCH (if
C     necessary).
C     The procedure of the data extraction provided by the
C     routine depends on the SPICE catalog type. If we deal
C     with type 2 it is relevant to refer to STGRD2 routine for
C     detail information.
C
C     This routine re-directs a "get position request" to the
C     appropriate type-specific routines; refer the type-specific
C     routine headers for detailed information regarding how request
C     is handled in each particular case.
C
C$ Examples
C   
C     C
C     C     This program is intended to demonstrate "read" interfaces
C     C     of the SPICE Sky Catalog System. 
C     C
C     C     In it we will be trying to read ACT Reference Catalog 
C     C     containing 988,758 stars covering the entire sky. This 
C     C     catalog has the following fields:
C     C
C     C        Field Name       FT Field Description
C     C        ---------------- -- ---------------------------------
C     C        AC2000           N  AC 2000 Number.
C     C    (*) BAND_NO          N  DEC band number.
C     C    (*) ZONE_IN_BAND_NO  N  Zone number within the DEC band.
C     C    (*) RA               N  RA offset from the zone center.  
C     C    (*) DEC              N  DEC offset from the zone center.  
C     C        BTMAG            N  Blue magnitude from Tycho
C     C        VTMAG            N  Visual magnitude from Tycho
C     C        B_V              N  Johnson B-V from Tycho
C     C        PMRA             N  PM in RA, (sec of time/yr)      
C     C        PMDEC            N  PM in Dec, (arcsec/yr)  
C     C        E_RA             N  St dev of RA at 1991.25 from Tycho
C     C        E_DEC            N  St dev of Dec at 1991.25 from Tycho
C     C        E_PMRA           N  St dev of Proper motion (RA)  
C     C        E_PMDEC          N  St dev of Proper motion (DE)  
C     C        FLAG1            C  Astrometric Quality flag
C     C        FLAG2            C  Quality flag
C     C        FLAG3            C  Known variability flag
C     C        FLAG4            C  Variability from Tycho
C     C        FLAG5            C  Duplicity from Tycho
C     C        TYC              C  Tycho ID, embedded zeros for blanks  
C     C        HIP              N  Hipparcos Number 
C     C        HD               N  Henry Draper Number
C     C        CPD              C  CPD identifier
C     C        CD               C  CD identifier
C     C        BD               C  BD identifier
C     C
C     C     Fields indicated by (*) are components of the internal 
C     C     object position representation in Type 2 SPICE Sky Catalog
C     C     system. They should not be accessed directly; the routine 
C     C     that returns object position, STGRD, should be used instead.
C     C     
C           PROGRAM READACT
C     
C           IMPLICIT NONE
C     
C     C
C     C     Local parameters.
C     C
C           INTEGER               MAXOBJ
C           PARAMETER           ( MAXOBJ = 400 )
C     
C           INTEGER               MAXVRT
C           PARAMETER           ( MAXVRT = 1 )
C     
C     
C           CHARACTER*(*)         CATFIL
C           PARAMETER           ( CATFIL = 'ACT.CAT' )
C           
C           CHARACTER*(*)         LSKFIL
C           PARAMETER           ( LSKFIL = 'naif0007.tls' )
C           
C     C
C     C     Local variables.
C     C
C           CHARACTER*(1)         F
C           CHARACTER*(80)        LINE
C     
C           DOUBLE PRECISION      A
C           DOUBLE PRECISION      BSIGHT ( 3 )
C           DOUBLE PRECISION      DEC
C           DOUBLE PRECISION      ET
C           DOUBLE PRECISION      RA
C           DOUBLE PRECISION      VERTCE ( 3, MAXVRT )
C     
C           INTEGER               I
C           INTEGER               J
C           INTEGER               LIST   ( MAXOBJ )
C           INTEGER               N
C           INTEGER               NSIDES
C     
C           LOGICAL               FOUND
C     
C     C
C     C     SPICELIB functions
C     C
C           DOUBLE PRECISION      DPR
C           DOUBLE PRECISION      HALFPI
C     
C     C
C     C     Load leapseconds file.
C     C
C           CALL FURNSH ( LSKFIL )
C     
C     C
C     C     Load ACT catalog in SPICE Sky catalog format.
C     C
C           CALL FURNSH ( CATFIL )
C     
C     C
C     C     Perform the first search to find all stars with visual 
C     C     magnitude between 4.5 and 6.3, good astrometric 
C     C     quality, and within 10 degrees of the north direction (+Z)
C     C     axis of B1950 inertial reference frame for OCT. 23, 2000 
C     C
C     
C     C
C     C     Set constraints for VTMAG value to find stars with visual 
C     C     magnitude between 4.5 and 6.3 (not including 6.3).
C     C
C           CALL STSCN ( 'VTMAG', 'LT', 6.3D0 )
C           CALL STSCN ( 'VTMAG', 'GE', 4.5D0 )
C     
C     C
C     C     Set constraint for FLAG2 (quality flag) to reject objects 
C     C     with low astrometric quality (FLAG2 = '7').
C     C
C           CALL STSCC ( 'FLAG2', 'NE', '7' )
C     
C     C
C     C     Define field of view as a circle with angular radius equal 
C     C     to 10 deg and boresight directed to the North pole in B1950 
C     C     frame. Note that we need two vectors -- boresight and a 
C     C     single vector pointing at any point on the circle -- to 
C     C     define a circular FOV.
C     C
C           NSIDES = 1
C           CALL VPACK ( 0.0D0, 0.0D0,              1.0D0,     BSIGHT )
C           CALL VPACK ( 0.0D0, DSIN(10.0D0/DPR()), DCOS(10.0D0/DPR()), 
C          .                                                   VERTCE )
C     
C     C     
C     C     Set FOV constraint.
C     C
C           CALL STSFOV( 0.0D0, 'B1950', 'CIRCLE', BSIGHT, NSIDES, 
C          .                                                   VERTCE )
C     
C     C
C     C     Set epoch for which star positions will be computed (or for 
C     C     which proper motion will be applied.)
C     C
C           CALL STR2ET ( '23 OCT 2000 00:00:00', ET )
C           CALL STSEPH ( ET )
C     
C     C
C     C     Print output table header.
C     C
C           WRITE(*,*)
C           WRITE(*,*)
C          .      ' Number    VTMag    FLAG3     AC2000      RA     DEC'
C     
C     C
C     C     Search for stars that meet all specified constraints.
C     C
C           N = 1
C     
C           DO WHILE ( N .GT. 0 )
C     
C     C
C     C        Return first/next portion (no more that MAXOBJ 
C     C        object numbers) of the list of stars that satisfy 
C     C        all constraints.
C     C
C              CALL STFTCH ( MAXOBJ, N, LIST )
C     
C     C
C     C        If returned portion of the list is not empty, get  
C     C        star parameters for each star.
C     C
C              IF ( N .GT. 0 ) THEN
C     
C                 DO I = 1, N
C     
C     C
C     C              First, get star position in J2000 inertial frame.
C     C
C                    CALL STGRD ( LIST(I), 'J2000',  RA,  DEC )
C     
C     C
C     C              Get 'VTMAG' -- visual magnitude. We don't need to
C     C              check FOUND flag because if VTMAG was one of the 
C     C              constrained fields, any objects with unavailable
C     C              VTMAG (or "null" VTMAG) will not present in the 
C     C              list.
C     C
C                    CALL STGFD ( LIST(I), 'VTMAG',  A, FOUND )
C     
C     C
C     C              Get 'AC2000' -- catalog ID of the star. For this 
C     C              field we also don't need to check FOUND flag 
C     C              because by definition objects ID cannot be "null".
C     C
C                    CALL STGFI ( LIST(I), 'AC2000', J, FOUND )
C     
C     C
C     C              Get 'FLAG3' -- known variability flag. Since this 
C     C              field was not constrained and it may be "null", 
C     C              so, we must check FOUND flag to make sure that 
C     C              non-"null" value was returned.
C     C
C                    CALL STGFC ( LIST(I), 'FLAG3',  F, FOUND )
C                    IF ( .NOT. FOUND ) THEN
C                       F = '-'
C                    END IF
C     
C     C
C     C              Print all data for this star.
C     C
C                    WRITE(*,'( I5, F12.3, 6X, A1, I13, 2F8.2 )')
C          .                    I, A, F, J, RA*DPR(), DEC*DPR()
C     
C                 END DO
C     
C              END IF
C     
C           END DO
C     
C     C
C     C     Clear constraints set for the first search to prepare for 
C     C     the second search.
C     C
C           CALL STCLRC ( )
C     
C     C
C     C     Set constraints for and perform the second search to find 
C     C     all bright stars on the sky by setting visual magnitude 
C     C     constraint at 3.0 for the same date (October 23, 2000).
C     C
C           CALL STSCN  ( 'VTMAG', 'LT', 3.0D0 )
C           CALL STSEPH ( ET )
C     
C     C
C     C     Print output table header and start search.
C     C
C           WRITE(*,*)
C           WRITE(*,*) 'Number   RA      DEC        VTMag'
C     
C           N = 1
C     
C           DO WHILE ( N .GT. 0 )
C     
C     C
C     C        Return first/next portion of the list of stars that 
C     C        satisfy all constraints.
C     C
C              CALL STFTCH ( MAXOBJ, N, LIST )
C     
C     C
C     C        If returned portion of the list is not empty, get  
C     C        star parameters for each star.
C     C
C              IF ( N .GT. 0 ) THEN
C     
C                 DO I = 1, N
C     
C     C
C     C              Get and print star position in J2000 inertial 
C     C              frame and visual magnitude.
C     C
C                    CALL STGRD ( LIST(I), 'J2000',  RA,  DEC )
C     
C                    CALL STGFD ( LIST(I), 'VTMAG',  A, FOUND )
C     
C                    WRITE( *, '( I5, 2F8.2, F12.3 )' )
C          .                    I, RA * DPR(), DEC * DPR(), A
C     
C                 END DO
C     
C              END IF
C     
C           END DO
C     
C     C
C     C     All done: unload the catalog ...
C     C
C           CALL UNLOAD ( CATFIL )
C     
C           END
C     
C     Output from this program will look like this:
C
C       Number    VTMag    FLAG3     AC2000      RA     DEC
C         1       5.233      G      1990924  115.13   87.02
C         2       5.587      -      2100176  351.75   87.31
C         3       5.711      -      1960967   53.08   84.91
C         4       5.878      -      1965474   62.51   86.63
C         5       5.549      -      2007841  157.42   84.25
C         6       5.791      -      2050286  262.70   86.97
C         7       5.261      -      2089092  333.29   86.11
C         8       6.050      -      2094843  342.76   85.37
C         9       4.853      -      2095327  343.60   84.35
C        10       5.601      -      1941811   13.72   83.71
C        11       6.062      -      1951158   32.36   81.30
C        12       5.931      -      1955794   41.95   81.45
C        13       5.931      N      1958511   47.93   81.47
C        14       5.489      -      1967577   67.06   83.81
C        15       5.565      -      1967760   67.50   83.34
C        16       5.242      -      1971197   75.09   81.19
C        17       6.284      -      1988980  111.34   81.26
C        18       5.076      G      1989736  112.77   82.41
C        19       6.282      -      1995931  126.14   82.43
C        20       6.162      N      2013204  172.96   81.13
C        21       6.153      -      2016754  182.75   81.71
C        22       5.809      -      2019630  192.28   83.42
C        23       5.314      -      2019646  192.31   83.41
C        24       6.027      -      2024593  205.60   82.75
C        25       5.710      -      2031255  222.59   82.51
C        26       6.167      -      2057844  276.04   83.18
C        27       5.482      -      2075547  307.06   81.42
C        28       6.213      -      2075653  307.26   83.63
C        29       6.071      -      2075704  307.36   81.09
C        30       5.748      N      2077176  310.65   82.53
C        31       6.130      -      2080632  318.34   81.23
C        32       4.919      -      2094295  341.87   83.15
C        33       5.534      -      1967459   66.76   80.82
C        34       5.893      -      2048862  259.90   80.14
C        35       5.731      -      2054132  270.04   80.00
C        36       5.489      N      2077753  311.89   80.55
C        37       6.069      -      2081919  321.21   80.52
C     
C      Number   RA      DEC        VTMag
C         1   95.99  -52.70      -0.608
C         2   88.79    7.41       0.769
C         3   68.98   16.51       1.160
C         4  344.41  -29.62       1.248
C         5   81.57   28.61       1.670
C         6  332.06  -46.96       1.757
C         7  122.38  -47.34       1.793
C         8  306.41  -56.74       1.898
C         9   99.43   16.40       2.024
C        10   86.94   -9.67       2.037
C        11  283.82  -26.30       2.044
C        12    2.10   29.09       2.050
C        13  252.17  -69.03       2.081
C        14  177.26   14.57       2.143
C        15  211.67  -36.37       2.190
C        16  233.67   26.71       2.217
C        17   17.43   35.62       2.264
C        18  340.67  -46.88       2.299
C        19  218.88  -42.16       2.306
C        20  305.56   40.26       2.307
C        21  265.62  -39.03       2.357
C        22  137.00  -43.43       2.408
C        23  178.46   53.69       2.431
C        24    6.57  -42.31       2.519
C        25  182.09  -50.72       2.551
C        26  221.25   27.07       2.622
C        27  224.63  -43.13       2.646
C        28  262.69  -37.30       2.651
C        29  189.30  -69.14       2.659
C        30   45.57    4.09       2.736
C        31  183.79  -58.75       2.755
C        32  200.15  -36.71       2.761
C        33  248.97  -28.22       2.793
C        34   74.25   33.17       2.857
C        35  275.25  -29.83       2.860
C        36  229.73  -68.68       2.884
C        37  296.56   10.61       2.889
C        38  265.87    4.57       2.899
C        39  245.30  -25.59       2.922
C        40  195.54   10.96       2.947
C        41  187.47  -16.51       2.959
C        42  261.32  -55.53       2.992
C        43  328.48  -37.36       2.994
C
C$ Restrictions
C
C     SPICE Sky catalog file must be loaded using STLOAD routine
C     prior to calling this routine.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     V.N. Kheifets  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 01-DEC-1999 (VNK)
C
C-&

C$ Index_Entries
C
C     return position of sky_catalog object
C
C-&

C
C     SPICELIB functions
C
      LOGICAL               RETURN

C
C     Local variables
C
      INTEGER               TYPE

C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'STGRD' )
      END IF

C
C     Get the catalog type and call corresponding type-specific
C     routine.
C
      CALL STTYPE ( TYPE )

      IF ( TYPE .EQ. 0 ) THEN

         CALL SETMSG ( 'Must load catalog before getting data.' )
         CALL SIGERR ( 'SPICE(CATALOGISNOTLOADED)'              )

      ELSE IF ( TYPE .EQ. 2 ) THEN

         CALL STGRD2 ( RECNO, FRAME, RA, DEC )

      ELSE

         CALL SETMSG ( 'Cannot work with catalog type #.' )
         CALL ERRINT ( '#', TYPE                          )
         CALL SIGERR ( 'SPICE(UNSUPPORTEDTYPE)'           )

      END IF

      CALL CHKOUT ( 'STGRD' )

      RETURN

      END
		
