C$Procedure   STI2F2 ( Unpacking a few integer number from one integer )

      SUBROUTINE STI2F2 ( K, N, A )

C$ Abstract
C
C     Unpack a few integer numbers from one integer and return them.
C
C$ Copyright
C
C     Copyright (2000), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     SPICE Sky Catalog Subsystem
C
C$ Keywords
C
C     TBD
C
C$ Declarations

      IMPLICIT              NONE

      INCLUDE               'starlib.inc'

      INTEGER               K
      INTEGER               N
      INTEGER               A (*)

C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     K          I   Integer number containing packed integers
C     N          I   Number of integers to be unpacked
C     A          O   Array of N unpacked integer numbers
C
C$ Detailed_Input
C
C     K              is the output integer representing N integers
C                    in packed form.
C
C     N              is the number of integer values to be packed.
C
C
C$ Detailed_Output
C
C     A              is an array of N integers that were unpacked 
C                    from K.
C
C$ Parameters
C
C     See include file.
C
C$ Exceptions
C
C     1) If the number of integers to be unpacked is less than
C        or equal to zero, then the error 'SPICE(TOOFEWINTEGERS)' 
C        will be signalled.
C     
C     2) If the number of integers to be unpacked is greater than
C        parameter NIIMAX, then the error 'SPICE(TOOMANYINTEGERS)' 
C        will be signalled.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     The input integer J contains N integer numbers packed using the
C     following scheme: 32-bit in J are divided into N M-bit (M=32/N)
C     subsets each representing an integer number. These subsets are
C     adjacent to each other (without gaps) and shifted right to the
C     least significant bit (see below):
C
C             31..............................0
C                   xxx...x ... xxx...x xxx...x
C                   \_____/     \_____/ \_____/
C                      M           M       M
C                     A(1)  ...  A(N-1)   A(N)
C
C$ Examples
C
C     Assume that we have K = -1059108013 (decimal) = C0DF4B53 (hex) 
C     and it contains N = 8 integer numbers in packed using Type 2 
C     SPICE Sky catalog approach. Then on calling
C
C             CALL STI2F2 ( K, 8, A )
C
C     we obtain the following values of integer numbers being unpacked
C     from K (i.e. elements of A array):
C
C             A(1)  = -3
C             A(2)  = -1
C             A(3)  =  2
C             A(4)  =  0
C             A(5)  = -5
C             A(6)  =  4
C             A(7)  = -6
C             A(8)  = -3
C
C$ Restrictions
C
C     The number of integers to be unpacked must to be positive and 
C     less than 17 (or NIIMAX+1).
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     V.N. Kheifets   ( JPL )
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 1-DECEMBER-1999 (VNK)
C
C-&

C$ Index_Entries
C
C     unpack few integer numbers from one for type_2 sky_catalog
C
C-&

C
C     Local variables
C
      INTEGER J
      INTEGER M
      INTEGER P
      INTEGER Q
      INTEGER I
      INTEGER Z
      INTEGER AI

C
C     SPICELIB functions
C
      LOGICAL               RETURN

C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'STI2F2' )
      END IF

C
C     The number of integers to be unpacked must be positive and
C     whether less than or equal to 16 (parameterized in NIIMAX.)
C
      IF ( N .LE. 0 ) THEN
         CALL SETMSG ( 'The number of integers to be unpacked ' //
     .                 'must be greater than zero. It was #.'   )
         CALL ERRINT ( '#', N                                   )
         CALL SIGERR ( 'SPICE(TOOFEWINTEGERS)'                  )
         CALL CHKOUT ( 'STI2F2'                                 ) 
         RETURN
      END IF

      IF ( N .GT. NIIMAX ) THEN
         CALL SETMSG ( 'The number of integers to be unpacked ' //
     .                 'must be less than #. It was #.'         )
         CALL ERRINT ( '#', N                                   )
         CALL ERRINT ( '#', NIIMAX                              )
         CALL SIGERR ( 'SPICE(TOOMANYINTEGERS)'                 )
         CALL CHKOUT ( 'STI2F2'                                 )
         RETURN
      END IF

C
C     If we need to unpack one integer from one integer, we simply 
C     return what we have on the input.
C
      IF ( N .EQ. 1 ) THEN
         A(1) = K
         CALL CHKOUT ( 'STI2F2' )
         RETURN
      END IF

C
C     Initialize auxiliary variables.
C
      M = 32 / N
      P = 2**M
      Q = P / 2
      J = K

C
C     Un-pack the numbers.
C
      DO I = 1, N

C
C        We "move" from the most significant bits to the least 
C        significant to extract integers from 1st to Nth.
C
         Z = 2**(M*(N-I))
         AI = J / Z

C
C        Re-assign J to be positive remainder. 
C
         J = IABS ( J - AI * Z )

C
C        If the number if greater than Q, it should be converted to
C        its negative complement.
C
         IF( AI .GT. Q ) THEN
            AI = AI - P
         END IF

         A(I) = AI

      END DO

      CALL CHKOUT ( 'STI2F2' )

      RETURN

      END
