C$Procedure  STI2T2 ( Unpack text string from integer array )

      SUBROUTINE STI2T2 ( PACKED, NPACK, TEXT, NTEXT )

C$ Abstract
C
C     Unpack text string from an integer array.
C
C$ Copyright
C
C     Copyright (2000), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     SPICE Sky Catalog Subsystem
C
C$ Keywords
C
C     TBD
C
C$ Declarations

      IMPLICIT              NONE

      INCLUDE               'starlib.inc'

      INTEGER               PACKED (*)
      INTEGER               NPACK
      CHARACTER*(*)         TEXT
      INTEGER               NTEXT

C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     PACKED     I   Integer array containing packed text
C     NPACK      I   Number of array elements containing packed text
C     TEXT       O   Text string unpacked from integer array
C     NTEXT      O   Number of characters that was unpacked
C
C$ Detailed_Input
C
C     PACKED         is integer array containing text string encoded 
C                    by integers.
C
C     NPACK          is the number of elements of array PACKED that 
C                    are used to contain packed string.
C
C$ Detailed_Output
C
C     TEXT           is a text string that was unpacked from integer
C                    array.
C
C     NTEXT          is the number of characters of TEXT that was 
C                    unpacked.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) If the number of elements to be unpacked is less than or 
C        equal to zero, then the error 'SPICE(TOOFEWINTSTOUNPACK)'
C        will be signalled.     
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     The input integer array is un-packed into the output string
C     array using the following schema:
C
C        -- each array element representing NCHARI integers packed 
C           using "few-integers-into-one-integer" Type 2 Sky catalog 
C           packing schema is unpacked to extract them;
C
C        -- each integer is converted to character using CHAR;
C
C     If output string size is smaller than NPACK * NCHARI, than only 
C     only number of characters equal to the string size will be 
C     unpacked.
C
C$ Examples
C
C     Let we have the following integer array:
C
C           INTEGER            J ( 5, 12 )
C
C           DATA    J 
C          .  / 
C          .  1281975909,  661856354, 1970544737,  544694636,
C          .  1802071655,  544434273, 1685026604,  543236208,
C          .  1869574688, 1886151033, 1701978144,  538976288,
C          .  1416126836,  544437362, 1970565920, 1634624544,
C          .  1718773108, 1931503721, 1931503727, 1970413685,
C          .  1886350880, 1952998688, 1937006951, 1696604192,
C          .  1097753632, 1952998766,  543781664, 1751474546,
C          .  1679847023,  544042866, 1698308201, 1948281203,
C          .   543236212, 1634493728,  538976288,  538976288,
C          .  1416588388,  543324448, 1634607209, 1684631412,
C          .   740320885, 1819025519, 1713402735, 1970168864,
C          .  1634624544, 1718973049,  740302880,  538976288,
C          .  1399416686, 1768323433, 1852252270, 1869899881,
C          .  1852255776,  538976288,  538976288,  538976288,
C          .   538976288,  538976288,  538976288,  538976288   
C          .  /
C
C     then this code:
C
C           DO K = 1, 5
C              CALL STI2T2 ( J, 12, S, M )
C              WRITE (*,*) S
C           END DO
C
C     where S is character string of at least 48 bytes, will print 
C     the following:
C
C          Life's but a walking shadow, a poor player
C          That struts and frets his hour upon the stage
C          And then is heard no more: it is a tale
C          Told by an idiot, full of sound and fury,
C          Signifying nothing.
C
C$ Restrictions
C
C     The number of elements to be unpacked must be greater than
C     zero.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     V.N. Kheifets   (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 1-DECEMBER-1999 (VNK)
C
C-&

C$ Index_Entries
C
C     unpack string from integers for type_2 sky_catalog
C
C-&

C
C     SPICELIB functions
C     
      LOGICAL               RETURN

C
C     Local variables
C
      INTEGER               Z ( NCHARI )
      INTEGER               I
      INTEGER               K
      INTEGER               L
      INTEGER               TXTLEN

C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'STI2T2' )
      END IF

C
C     Check whether number of elements makes sense.
C
      IF ( NPACK .LE. 0 ) THEN
         CALL SETMSG ( 'Specified number of array elements from ' //
     .                 'which string should be unpacked was #. '  //
     .                 'It must be greater zero. '                )
         CALL ERRINT ( '#', NPACK                                 )
         CALL SIGERR ( 'SPICE(TOOFEWCINTSTOUNPACK)'               )
         CALL CHKOUT ( 'STT2I2'                                   )
         RETURN
      END IF

C
C     Memorize size of the output string.
C
      TXTLEN = LEN( TEXT )

C
C     Determine number of packed characters.
C
      NTEXT = NPACK * NCHARI

C
C     Loop to unpack each integer in PACKED. 
C
      L = 0

      DO I = 1, NPACK

         CALL STI2F2 ( PACKED(I), NCHARI, Z )

         DO K = 1, NCHARI

            L = L + 1

C
C           We don't want to go beyond string end. 
C
            IF ( L .LE. TXTLEN ) THEN
               TEXT(L:L) = CHAR ( Z(K) )
            END IF

         END DO

      END DO

      CALL CHKOUT ( 'STI2T2' )

      RETURN

      END
