C$Procedure      STLMGR ( SPICE Sky Catalog, load/unload manager)

      SUBROUTINE STLMGR ( NAME, TYPE )

C$ Abstract
C
C     Manage load/unload high level operations on SPICE Sky Catalog
C     files.
C
C$ Copyright
C
C     Copyright (2000), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     SPICE Sky Catalog Subsystem
C
C$ Keywords
C
C     TBD
C
C$ Declarations

      IMPLICIT NONE

      INCLUDE               'starlib.inc'

      CHARACTER*(*)         NAME
      INTEGER               TYPE

C$ Brief_I/O
C
C     VARIABLE  I/O  ENTRY POINTS
C     --------  ---  --------------------------------------------------
C     NAME       I   STLOAD
C     TYPE       O   STTYPE
C
C$ Detailed_Input
C
C     NAME           is catalog file name
C
C$ Detailed_Output
C
C     TYPE           is an integer code of the catalog type. If no
C                    catalog is loaded then TYPE is zero.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     See entry point headers for exception that handled by the
C     entry points.
C
C     1) If the umbrella routine is called, then the error
C        'SPICE(BOGUSENTRY)' will be signalled.
C
C$ Files
C
C     TBD
C
C$ Particulars
C
C     STLMGR is an umbrella routine for its entry points:  all
C     variables used by the entry points are declared here.
C
C     STLMGR entry points load and unload SPICE Sky Catalog files
C     and return type of a currently loaded catalog. The entry
C     points and their functions are:
C
C        Catalog loading and unloading:
C
C           STLOAD  ( load catalog     )
C           STUNLD  ( unload catalog   )
C
C        Type extraction:
C
C           STTYPE  ( get catalog type )
C
C     Local variable CTYPE describes loaded SPICE catalog qualitatively
C     and quantitatively as well. Qualitatively  - if no catalog is
C     loaded, CTYPE is equal to 0. Quantitatively - if CTYPE is
C     distinct from zero, its value is the catalog type.
C
C$ Examples
C
C     To load sky catalog file 'act.bsk', call STLOAD as follows:
C
C           CALL STLOAD ( 'act.bsk' )
C
C     To unload currently loaded catalog file, call STUNLD as 
C     follows:
C
C           CALL STUNLD ( )
C
C$ Restrictions
C
C     See entry point headers.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     V.N. Kheifets  (JPL)
C
C$ Version
C
C-    SPICELIB pre-release Version 1.0.1, 20-JAN-2000 (VNK)
C
C-&

C
C     SPICELIB functions
C
      LOGICAL               RETURN

C
C     Local variables
C
      INTEGER               CTYPE
      INTEGER               HANDLE
      CHARACTER*(3)         SPARCH
      CHARACTER*(3)         SPTYPE

C
C     Save all local variables
C
      SAVE

C
C     Initial values
C
      DATA            CTYPE   / 0 /

C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'STLMGR' )
      END IF

C
C     Never come here.
C
      CALL SIGERR ( 'SPICE(BOGUSENTRY)' )
      CALL CHKOUT ( 'STLMGR' )

      RETURN

C$Procedure STLOAD ( SPICE Sky Catalog Subsystem, load catalog )

      ENTRY STLOAD ( NAME )

C$ Abstract
C
C     High-level routine to load catalog
C
C$ Copyright
C
C     Copyright (2000), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     SPICE Sky Catalog Subsystem
C
C$ Keywords
C
C     TBD
C
C$ Declarations
C
C     CHARACTER*(*)         NAME
C
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     NAME       I   The catalog filename
C
C$ Detailed_Input
C
C     NAME           is the name of file with the SPICE catalog
C
C$ Detailed_Output
C
C     None.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) If another catalog is loaded at the time when this routine
C        is called, then the error 'SPICE(CATALOGALREADYLOADED)' will
C        be signalled.
C
C     2) If the type of the catalog that is being loaded is not
C        supported, then the error 'SPICE(UNKNOWNCATALOGTYPE)' will
C        be signalled.
C
C$ Files
C
C     TBD.
C
C$ Particulars
C
C     None.
C
C$ Examples
C
C     To load sky catalog file 'act.bsk', call STLOAD as follows:
C
C           CALL STLOAD ( 'act.bsk' )
C
C$ Restrictions
C
C     None
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     V.N. Kheifets  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 01-DEC-1999 (VNK)
C
C-&

C$ Index_Entries
C
C     load sky_catalog file
C
C-&

C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'STLOAD' )
      END IF

C
C     Check whether some catalog is already loaded.
C
      IF ( CTYPE .NE. 0 ) THEN

         CALL SETMSG ( 'Cannot load catalog because some other'   //
     .                 'catalog is already loaded. '              )
         CALL SIGERR ( 'SPICE(CATALOGALREADYLOADED)'              )

      ELSE

C
C        Determine catalog file architecture and type.
C
         CALL GETFAT ( NAME, SPARCH, SPTYPE )

C
C        Check whether architecture and type are recognized and
C        supported.
C
         IF ( SPARCH .EQ. 'DAS' .AND. SPTYPE .EQ. ST2TYP ) THEN

C
C           This is type 2 catalog; proceed to load ...
C
            CTYPE = 2
            CALL DASOPR ( NAME, HANDLE )
            CALL STLD2  ( HANDLE )

         ELSE

            CTYPE = 0
            CALL SETMSG ( 'Cannot recognize type of the catalog ' //
     .                    '''#''. Architecture/type were '        //
     .                    '''#''/''#''. '                           )
            CALL ERRCH  ( '#', NAME                                 )
            CALL ERRCH  ( '#', SPARCH                               )
            CALL ERRCH  ( '#', SPTYPE                               )
            CALL SIGERR ( 'SPICE(UNKNOWNCATALOGTYPE)'               )

         END IF

      END IF

      CALL CHKOUT ( 'STLOAD' )

      RETURN


C$Procedure STUNLD ( SPICE Sky Catalog Subsystem, unload catalog )

      ENTRY STUNLD

C$ Abstract
C
C     High-level routine to close file with the SPICE catalog
C
C$ Copyright
C
C     Copyright (2000), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     SPICE Sky Catalog Subsystem
C
C$ Keywords
C
C     TBD
C
C$ Declarations
C
C     None.
C
C$ Brief_I/O
C
C     None.
C
C$ Detailed_Input
C
C     None.
C
C$ Detailed_Output
C
C     None.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) If no catalog is loaded at the time when this routine is
C        called, then the error 'SPICE(CATALOGISNOTLOADED)' will
C        be signalled.
C
C$ Files
C
C     TBD.
C
C$ Particulars
C
C     None.
C
C$ Examples
C
C     To unload currently loaded catalog file, call STUNLD as 
C     follows:
C
C           CALL STUNLD ( )
C
C$ Restrictions
C
C     SPICE Sky catalog file must be loaded using STLOAD routine
C     prior to calling this routine.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     V.N. Kheifets  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 01-DEC-1999 (VNK)
C
C-&

C$ Index_Entries
C
C     unload sky_catalog file
C
C-&

C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'STUNLD' )
      END IF

C
C     Check whether some catalog is really loaded.
C
      IF ( CTYPE .EQ. 0 ) THEN

         CALL SETMSG ( 'No catalog is loaded -- cannot unload.' )
         CALL SIGERR ( 'SPICE(CATALOGISNOTLOADED)'              )
      
      ELSE

C
C        Check the catalog type and unload using appropriate 
C        routines.
C
         IF ( CTYPE .EQ. 2 ) THEN
            CALL STULD2 ( HANDLE )
            CALL DASCLS ( HANDLE )
         END IF

      END IF

      CTYPE = 0

      CALL CHKOUT( 'STUNLD' )

      RETURN


C$Procedure STTYPE ( SPICE Sky Catalog Subsystem, get catalog type )

      ENTRY STTYPE ( TYPE )

C$ Abstract
C
C     High-level routine returning the type of a loaded catalog.
C
C$ Copyright
C
C     Copyright (2000), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     SPICE Sky Catalog Subsystem
C
C$ Keywords
C
C     TBD
C
C$ Declarations
C
C     INTEGER               TYPE
C
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     TYPE       O   Integer code of the catalog type
C
C$ Detailed_Input
C
C     None
C
C$ Detailed_Output
C
C     TYPE           is an integer code of the catalog type. If no
C                    catalog is loaded then TYPE is zero.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     None.
C
C$ Examples
C
C     None.
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     V.N. Kheifets  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 01-DEC-1999 (VNK)
C
C-&

C$ Index_Entries
C
C     return type of loaded sky_catalog file
C
C-&

C
C     Return the catalog type
C
      TYPE = CTYPE

      RETURN

      END
		
