C$Procedure   STMPK2 ( Determine packing density for a given integer )

      INTEGER FUNCTION STMPK2 ( J )

C$ Abstract
C
C     Determine the packing density -- the maximum number of integers
C     with the absolute value less than or equal to J that can be
C     "packed" into a single 32-bit integer using type 2 Sky Catalog
C     integer packing approach.
C
C$ Copyright
C
C     Copyright (2000), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     SPICE Sky Catalog Subsystem
C
C$ Keywords
C
C     TBD
C
C$ Declarations

      INTEGER               J

C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     J          I   Positive integer number
C     STMPK2     O   Packing density
C
C$ Detailed_Input
C
C     J              is a positive integer number. 
C
C$ Detailed_Output
C
C     STMPK2         is the packing density -- the maximum number 
C                    of integers with the absolute value less of equal
C                    to J that can be "packed" into a single 32-bit
C                    integer using type 2 Sky Catalog integer packing
C                    approach.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) If input value J is less or equal to zero, then the error
C        'SPICE(WRONGARGUMENTVALUE)' will be signalled.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     According Type 2 SPICE Sky Catalog integer packing scheme, 32-bit
C     integer number, considered as a "package", is divided into N M-bit
C     subsets without gaps in such a way that these NxM bits are
C     shifted right to the least significant bit (see below):
C
C            31..............................0
C                  xxx...x ... xxx...x xxx...x
C                  \_____/     \_____/ \_____/
C                     M           M       M
C                    A(1)  ...  A(N-1)   A(N)
C
C     The task is to compute maximum number N by given maximal absolute
C     value of A(i)
C
C$ Examples
C
C     Below we present a set of pairs "input - output" corresponding to
C     calling sequence I = STMPK2 ( J ):
C
C                    J (input)    I (output)
C                    ----------   ----------
C                         1          16
C                         2          10
C                         3          10
C                         4           8
C                       ...           8
C                         7           8
C                         8           6
C                       ...           6
C                        15           6
C                        16           5
C                       ...           5
C                        31           5
C                        32           4
C                       ...           4
C                       127           4
C                       128           3
C                       ...           3
C                       511           3
C                       512           2
C                       ...           2
C                     32767           2
C                     32768           1
C                       ...           1
C
C$ Restrictions
C
C     The input of this routine cannot be zero or a negative number.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     V.N. Kheifets  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 01-DEC-1999 (VNK)
C
C-&

C$ Index_Entries
C
C     determine integer packing density for type_2 sky_catalog
C
C-&

C
C     Local variables
C
      INTEGER         I
      INTEGER         K

C
C     Do discovery "check-in" for the input parameter -- it must be 
C     a positive number.
C
      IF ( J .LE. 0 ) THEN
         STMPK2 = 1
         CALL CHKIN  ( 'STMPK2'                                  )
         CALL SETMSG ( 'The input was not a positive number: #.' )
         CALL ERRINT ( '#', J                                    )
         CALL SIGERR ( 'SPICE(WRONGARGUMENTVALUE)'               )
         CALL CHKOUT ( 'STMPK2'                                  )
         RETURN
      END IF

C
C     If the number requires 15 or more bits, the density is 1.
C
      IF ( J .GE. 32768 ) THEN
         STMPK2 = 1
         RETURN
      END IF

C
C     Otherwise, we need to compute the density. 
C 
      I = 1
      K = 1
      DO WHILE ( I .LE. J )
         I = I * 2
         K = K + 1
      END DO

      STMPK2 = 32 / K

      RETURN

      END
