C$Procedure STPCK2 ( Determine optimal packing catalog numeric fields )

      SUBROUTINE STPCK2 ( FLDNO, MAXABS, N, NPACK, DENSTY )

C$ Abstract
C
C     Determine optimal compression attributes for a set of numeric
C     fields using Type 2 Sky catalog packing scheme and re-order
C     fields to put them in increasing maximum absolute value order.
C
C$ Copyright
C
C     Copyright (2000), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     SPICE Sky Catalog Subsystem
C
C$ Keywords
C
C     TBD
C
C$ Declarations

      IMPLICIT NONE

      INCLUDE               'starlib.inc'

      INTEGER               FLDNO   (*)
      INTEGER               MAXABS  (*)
      INTEGER               N
      INTEGER               NPACK
      INTEGER               DENSTY (*)

C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     FLDNO     I/O  Array of numerical field indexes
C     MAXABS     I   Array of maximum absolute values for fields
C     N          I   Number of elements in FLDNO and MAXABS
C     NPACK      O   Number of packages needed to hold packed values
C     DENSTY     O   Array of packing densities for each package
C
C$ Detailed_Input
C
C     FLDNO          is an integer array containing indexes of a  
C                    set of catalog numeric fields.
C
C     MAXABS         is an integer array containing maximum absolute
C                    values for each numeric field from FLDNO.
C
C     N              is the number of elements in FLDNO and MAXABS 
C                    arrays.
C
C$ Detailed_Output
C
C     FLDNO          is an integer array containing indexes of a  
C                    set of catalog numeric fields re-ordered by
C                    increasing MAXABS.
C
C     NPACK          is the number of packages (integer numbers)
C                    needed to hold with packed fields values.
C
C     DENSTY         is an integer array of NPACK elements, 
C                    each element of which contains packing density, 
C                    the number of numeric field values that can be 
C                    stored in a single integer number.
C
C$ Parameters
C
C     See include file.
C
C$ Exceptions
C
C     1) If any of the MAXABS elements is zero or a negative number,
C        then an error will be be signalled by the STMPK2 function
C        called by this routine.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     This routine determines optimal way in which values of a number
C     of numeric fields can be packed into as a set integer numbers
C     using Type 2 SPICE Sky catalog numeric field packing approach.
C     The routine does this in two steps:
C
C        1.   Integer array of numerical fields indexes is sorted by
C             MAXABS increasing, where MAXABS - absolute maximum values
C             of the fields (in integer presentation)
C
C        2.   The sequences of "packages" (integer numbers) with 
C             maximal compression are determined by the sorted array.
C
C     The second step is very easy and evident. On sorting the array by
C     MAXABS increasing, its elements (MAXABS) are looked through from
C     left to right. For each field we determine the maximal packing
C     density Mmax by MaxAbs as an argument:
C
C                                                       M-1
C       Mmax is minimal value of M meeting requirement 2    > MaxAbs
C
C     When value of Mmax becomes less or equal than current element
C     number it means that the current "package" is complete and we
C     are to collect the next package starting with the next element.
C
C     Upon completion, STPCK2 routine generates the following output
C     information related to optimal packing:
C
C        --   integer array FLDNO which elements are field numbers
C             sorted by MAXABS increasing,
C
C        --   the number of "packages" NPACK (the number of
C             integers containing numerical field values),
C
C        --   integer array DENSTY of NPACK elements presenting
C             packing density - number of packed fields in different
C             packages (by definition the array is monotone
C             decreasing).
C
C     The FLDNO and DENSTY arrays data give us opportunity to find
C     out what numerical fields are packed in certain package and in
C     what sequence.
C
C$ Examples
C
C     Suppose we have integer array FLD(10) containing numerical
C     fields indexes and integer array MAXABS(10) with corresponding
C     field maximum absolute values. The values of the arrays elements
C     are presented below (see "INPUT DATA"). By calling
C
C           CALL STPCK2 ( FLD, MAXABS, 10, NPACK, DENSTY )
C
C     we obtain the number of integer "packages" NPACK = 5 and the
C     following elements of output array FLD and output array DENSTY
C     (see "OUTPUT DATA").
C
C                INPUT  DATA                     OUTPUT  DATA
C
C              FLD        MAXABS               FLD      DENSTY
C
C              17           155                 32         4
C              32             8                 14         2
C               4          6240                 11         2
C              22           712                  3         1
C               5           345                 17         1
C              11            45                  5
C               6     210000000                 22
C               3           108                 23
C              14            22                  4
C              23          1024                  6
C
C     It means that the first integer package contains 4 packed fields,
C     namely 32, 14, 11 and 3; the second package contains 2 fields:
C     17 and 5, Then, third package contains 2 fields too: 22 and 23,
C     and, at last, the fourth and fifth packages consists of one field
C     each: correspondingly 4 and 6.
C
C$ Restrictions
C
C     No element of MAXABS can be zero or a negative number.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     V.N. Kheifets  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 01-DEC-1999 (VNK)
C
C-&

C$ Index_Entries
C
C     determine optimal packing density for type_2 sky_catalog
C
C-&

C
C     Local variables
C
      INTEGER               B      ( NFLMAX )
      INTEGER               RNK    ( NFLMAX )
      INTEGER               I
      INTEGER               M
      INTEGER               K
      INTEGER               FDNSTY

C
C     SPICELIB functions
C
      INTEGER               STMPK2
      LOGICAL               RETURN

C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'STPCK2' )
      END IF

C
C     Re-order field indexes array FLDNO and a copy of MAXABS by
C     increasing MAXABS.
C
      CALL MOVED  ( MAXABS, N, RNK )
      CALL ORDERI ( MAXABS, N, B )
      CALL REORDI ( B, N, FLDNO  )
      CALL REORDI ( B, N, RNK    )

C
C     Determine the number of packages and capacity of each package.
C     The exit from this "run forever" loop is through the RETURN when
C     we are done processing all fields.
C
C     K is the index of the first field that will be packed in into the
C     NPACK-th "package".
C
      K = 1
      NPACK = 1

      DO WHILE ( .TRUE. )

C
C        Let's find out how many fields we can pack into the next
C        "package". M is how many is still unprocessed, I is how many
C        the current package will hold. We start with I=1.
C
         M = N-K+1
         I = 1
         FDNSTY = STMPK2( RNK(K+I-1) )

C
C        Increase the number of fields in the package until we have
C        processed all left-over fields or current field packing
C        density is smaller than the number of fields that we have
C        packed into the package to this point.
C
         DO WHILE ( I .LT. M .AND. FDNSTY .GT. I )
            I = I + 1
            FDNSTY = STMPK2( RNK(K+I-1) )
         END DO

C
C        In cases when previous DO WHILE loop terminated because of
C        individual packing density was less than I, we need to adjust
C        the number of fields in the current package to exclude that
C        last field.
C
         IF ( FDNSTY .LT. I .AND. I .GT. 1 ) THEN
            I = I-1
         END IF
         
C
C        That's it, we have got I, the density of fields in current
C        package. If we don't have anymore fields left, we can
C        peacefully return.
C
         DENSTY ( NPACK ) = I

         IF ( K+I-1 .EQ. N ) THEN
            CALL CHKOUT ( 'STPCK2' )
            RETURN
         END IF

C
C        We are done with current "package" but still have some fields
C        to process. "Shift" the first unprocessed field index,
C        increase "package" counter and off to do more work.
C
         K = K+I
         NPACK = NPACK+1

      END DO

      RETURN

      END
