C$Procedure  STT2I2 ( Packing text string into integer numbers )

      SUBROUTINE STT2I2 ( TEXT, NTEXT, PACKED, NPACK )

C$ Abstract
C
C     Pack text string into integer numbers.
C
C$ Copyright
C
C     Copyright (2000), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     SPICE Sky Catalog Subsystem
C
C$ Keywords
C
C     TBD
C
C$ Declarations

      IMPLICIT              NONE

      INCLUDE               'starlib.inc'

      CHARACTER*(*)         TEXT
      INTEGER               NTEXT
      INTEGER               PACKED (*)
      INTEGER               NPACK

C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     TEXT       I   Text string to be packed
C     NTEXT      I   Number of characters from string to be packed
C     PACKED     O   Integer array containing the packed text
C     NPACK      O   Number of PACKED elements that were used
C
C$ Detailed_Input
C
C     TEXT           is a text string to be converted to integers
C
C     NTEXT          is the number of characters of string TEXT to 
C                    to be packed.
C
C$ Detailed_Output
C
C     PACKED         is integer array containing TEXT string encoded 
C                    by integers. It must be big enough to hold 
C                    packed TEXT.
C
C     NPACK          is the number of elements of PACKED array used 
C                    to hold packed TEXT.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) If the number of characters to be packed is less than or 
C        equal to zero, then the error 'SPICE(TOOFEWCHARSTOPACK)'
C        will be signalled.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     The input character string is packed into the output integer
C     array using the following schema:
C
C        -- each character is converted to integer using ICHAR;
C
C        -- each block of NCHARI (see parameters) integers is
C           packed into a single integer using "few-integers-
C           into-one-integer" Type 2 Sky catalog packing schema.
C
C     If the number of characters to be packed is greater than the 
C     input string length, then any character beyond the string 
C     end will be considered to be and be packed as space (' ').
C
C$ Examples
C
C     Let we have the following character string to be encoded:
C     
C           CHARACTER*(48)        S (5)
C
C           S(1) = 'Life''s but a walking shadow, a poor player  '
C           S(2) = 'That struts and frets his hour upon the stage'
C           S(3) = 'And then is heard no more: it is a tale      '
C           S(4) = 'Told by an idiot, full of sound and fury,    '
C           S(5) = 'Signifying nothing.                          '
C
C     Then, this fragment of code
C
C           DO K = 1, 5
C              CALL STT2I2 ( S(K), 48, J, M )
C              WRITE( *,'(4I12)')  ( J(K), K = 1, M )
C           END DO
C
C     where J is an integer array of at least 12 elements,
C     will produce the following output:
C
C         1281975909   661856354  1970544737   544694636
C         1802071655   544434273  1685026604   543236208
C         1869574688  1886151033  1701978144   538976288
C         1416126836   544437362  1970565920  1634624544
C         1718773108  1931503721  1931503727  1970413685
C         1886350880  1952998688  1937006951  1696604192
C         1097753632  1952998766   543781664  1751474546
C         1679847023   544042866  1698308201  1948281203
C          543236212  1634493728   538976288   538976288
C         1416588388   543324448  1634607209  1684631412
C          740320885  1819025519  1713402735  1970168864
C         1634624544  1718973049   740302880   538976288
C         1399416686  1768323433  1852252270  1869899881
C         1852255776   538976288   538976288   538976288
C          538976288   538976288   538976288   538976288
C
C$ Restrictions
C
C     The number of characters to be packed must be greater than
C     zero.
C
C     Output array PACKED must be big enough to hold packed text.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     V.N. Kheifets   (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 1-DECEMBER-1999 (VNK)
C
C-&

C$ Index_Entries
C
C     pack string into integers for type_2 sky_catalog
C
C-&

C
C     SPICELIB functions
C
      LOGICAL               RETURN
      INTEGER               STF2I2
      INTEGER               RTRIM

C
C     Local variables
C
      INTEGER               Z ( NCHARI )
      INTEGER               I
      INTEGER               K
      INTEGER               L
      INTEGER               M
      CHARACTER             T

C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'STT2I2' )
      END IF

C
C     Check whether number of characters to be packed makes 
C     sense.
C
      IF ( NTEXT .LE. 0 ) THEN
         CALL SETMSG ( 'Specified number of characters to '       //
     .                 'be packed was #. It must be greater '     //
     .                 'zero. String was ''#''.'                  )
         CALL ERRINT ( '#', NTEXT                                 )
         CALL ERRCH  ( '#', TEXT                                  )
         CALL SIGERR ( 'SPICE(TOOFEWCHARSTOPACK)'                 )
         CALL CHKOUT ( 'STT2I2'                                   )
         RETURN
      END IF

C
C     Determine number of integers that will be used to hold 
C     packed string.
C
      NPACK = NTEXT / NCHARI
      IF ( MOD( NTEXT, NCHARI ) .NE. 0 ) THEN
         NPACK = NPACK + 1
      END IF

C
C     Get position of the last non-blank character.
C
      M = RTRIM ( TEXT )

C
C     Loop to pack each of NPACK integer numbers.
C
      L = 0

      DO I = 1, NPACK

C
C        Each integer will contain four characters.
C        
         DO K = 1, NCHARI

            L = L + 1

C
C           Is this character an actual character from the 
C           string? If not, set it to blank. 
C
            IF ( L .LE. M ) THEN
               T = TEXT(L:L)
            ELSE
               T = ' '
            END IF

C
C           Store character index in the intermediate four-element
C           integer array.
C
            Z(K) = ICHAR(T)

         END DO

C
C        Pack intermediate array into a single integer.
C
         PACKED(I) = STF2I2( NCHARI, Z )

      END DO

      CALL CHKOUT ( 'STT2I2' )

      RETURN

      END
